#include <stdio.h>
#include <string.h>
#include <math.h>
#include "response.h"
#include "resp_info.h"
#include "ucsb.h"
#define NA "N/A"
#define VMS "V/m/s"

PSSTR * alloc_psstr() {
  PSSTR *p;

  if(!(p=(PSSTR *) calloc(1,sizeof(PSSTR)))) syserr("allocating PSSTR","",0);
  return(p);
}
int make_id_calcs(RESPINFO * r)
{
  PSSTR *p;
  char l[128];
  double fi, pi2fo;
  float restmp;
  /* 
    local version of r->Rdeo, to be used for calcs.  This is a bug
    workaround for multiple table entries.  First time through would 
    print N/A (correct), but would print Open (incorrect) on second 
    pass, because r->Rdeo was set to an arbitrarily large number for
    calculations

    r->Rdeo should not be used for calculations in the program at all.
    All calculations should be done here.
  */
  float Rdeo;

  /* need to make sure that Fid is calculated here */
  if(r->Tid) r->Fid = 1.0/r->Tid;
  if(!r->Tid && r->Fid)  r->Tid = 1.0/r->Fid;

  if(!(r->psstr = alloc_psstr())) exit(0);
  p = r->psstr;

  if(!r->year && !r->month && !r->day && r->dat){
    sscanf(r->dat,"%2d%2d%2d",&(r->year),&(r->month),&(r->day));
  }

  /* if series array vals are zero, leave them, probably correct */

  /* Rd, figure out display and calculation values */
  /* Rdeo is a special case, there are several possiblities */
  /* need to assume its open if zero for calcs, but want to show N/A */
  /* Atten and Guid parallel this */
  if(!r->Rdeo){
    p->Rdeov = strdup("N/A");
    Rdeo = 100000000.0;
  }
  else if(r->Rdeo >= 2.0e+6){
    p->Rdeov = strdup("Open");
    Rdeo = 100000000.0;
  }
  else {
    sprintf(l,"%.0f",r->Rdeo);
    p->Rdeov = strdup(l);
    Rdeo = r->Rdeo;
  }

  /* error check the GNid value */
  if(!r->GNid){
    fprintf(stderr,"GNid == 0.0\n");
    r->GNid = 1.0;
    load_pserr_str("Amplitude info is bad due to bad observed value for GNid (GNid <= 0.0)");
    /* print out an error message for the PS sheet */
  }
  else if(r->GNid < 0.0){
    fprintf(stderr,"GNid < 0.0\n");
    r->GNid *= -1.0; 
    load_pserr_str("Amplitude info is bad due to bad observed value for GNid (GNid <= 0.0)");
  }
  else{
    load_pserr_str("");
  }

  /* check and see if user is passing in Vext or Iext */
  /* r->Iext is expected in milliamps, the conversion to amps is later */
  if(!r->Iext && r->Vext && r->Rmeas){
    r->Iext = r->Vext / r->Rmeas;
  }
  /* some prelim calcs and prep for calculating Gid */
  r->Atten = Rdeo/(r->Rc + Rdeo + r->Rdsr1 + r->Rdsr2);
  /* if Isig does not exist, calculate from Iext */
  if(!r->Isig) r->Isig = r->Iext * r->Atten;

  /* recalculate Gd if we have all the necessary numbers */
  if(r->Gmfgr && r->Atten) r->Gd = r->Gmfgr * r->Atten;

  if(!cal && r->Mmfgr && (r->Isig || r->Iext) && r->GNid){
    r->Gid = sqrt(((r->Mmfgr*r->GNid)/(r->Iext/1000.)));
    if(v) fprintf(stderr,"calculated sig coil Gen const r->Gid: %g\n",r->Gid);
  }
  else if (cal && r->Cmfgr && (r->Isig || r->Iext) && r->GNid){
    /* calculation for using cal constant */
    /* will not assume cal coil use just because cal constant is provided */
    r->Gid =  ((r->Mmfgr*r->GNid)/((r->Iext/1000.0)*r->Cmfgr));
    if(v) fprintf(stderr,"calculated cal coil Gen const r->Gid: %g\n",r->Gid);
  }
  else {
    fprintf(stderr,"id_calcs(): not enough information available to complete ");
    fprintf(stderr,"calculations\n");
    fprintf(stderr,"cal %d, r->Mmfgr %g, r->Iext %g, ",cal,r->Mmfgr,r->Iext);
    fprintf(stderr,"r->Isig %g, r->Cmfgr %g, r->GNid %g\n",r->Isig,r->Cmfgr,r->GNid);
  }

  /* set Guid if we have attenuation value */
  r->Guid = r->Gid/r->Atten;

  /*
    This fork decides which of the two possible G's will be used for 
    the main output.  r->G is set to either r->Gd (calced from mfgr)
    or r->Gid (from id vals and Iext)

    r->GNid will have been modified to produce a viable value above.
    Use the originally read in value for GNid stored in r->GNid.

    r->G is used for the display of the SAC transfer function. 
    Need the closest thing to a good value for display and the 
    SAC transfer command info.

    This value is also spit out to the response generated output table.

    The values for Gid and Gd are maintained intact so that the 
    operator can make decisions about any problems.

  */
  if(r->GNid <= 0.0){
    if(v) fprintf(stderr,"Bad value for GNid %.4f ...  ",r->GNid);
    if(r->Gd) {
      if(v) fprintf(stderr,"using calculated r->Gd %g instead of bad measured r->Gid %g\n",r->Gd,r->Gid);
      r->G = r->Gd;
    }
    else {
      if(v) fprintf(stderr,"\n");
      /* use the faked value, there should already be an error str */
      r->G = r->Gid;
      /*
      return(-1);
      */
    }
  }
  else {
    r->G = r->Gid;
  }

  /*
    do some error checking on the Resistance values.

    Computed value for Rmeas should be within 10%
    Rmeas shouldn't be greater than Rc, allow 1% tolerance. 
  */
  if( r->Rc && r->Rmeas && r->Rdeo ){
    restmp = r->Rmeas * ( 1.0/r->Rc + 1.0/r->Rdeo );
    if( restmp > 1.10 || restmp < 0.90){
      fprintf(stderr,"Warning: resistances conflict: Rc %g, Rdeo %g, Rmeas %g\n",r->Rc,r->Rdeo,r->Rmeas);
    }
  }
  else if( r->Rc && r->Rmeas ){
    restmp = r->Rc/r->Rmeas;
    if( restmp < .99 ){
      fprintf(stderr,"Warning: resistances conflict: Rc %g, Rdeo %g, Rmeas %g\n",r->Rc,r->Rdeo,r->Rmeas);
    }
    if( restmp > 1.05 ){
      fprintf(stderr,"Warning: resistances conflict: Rc %g, Rdeo %g, Rmeas %g\n",r->Rc,r->Rdeo,r->Rmeas);
    } 
  }

  /* 
     Try to calculate a Damping value if not provided directly by user.
     This formula is not really very accurate, there is a constant involved
     that is different for every sensor type.  Usually Dcalc will be fed
     in by the user.  We also dont care too much about the actual damping
     because the llnlid output should be very accurate, the number that is
     output doesnt need any modifications 
  */
  if( ! r->Dcalc && r->Dmfgr && r->Atten ){
    /* not sure that this is correct */
    r->Dcalc = r->Dmfgr / r->Atten;
  }

  /* 
    do poles and zeros calculations 

    s1 and s2 are the poles
    s3 and s4 are the zeroes
  */

  pi2fo = M_PI * 2.0 * r->Fid;
  fi = (r->Did<=1.00) ? sqrt((1.0-(r->Did * r->Did))):sqrt((r->Did*r->Did)-1.0);

  r->paz[0].r = r->paz[1].r = pi2fo * -1.0000 * r->Did;
  r->paz[0].i = pi2fo * fi;
  r->paz[1].i = pi2fo * fi * -1.00000;
  r->paz[2].r = r->paz[2].i = r->paz[3].r = r->paz[3].i =  0.000000;

  /* display strings */
  sprintf(l,"%.0f",r->Rdsr1);
  p->Rdsr1v = strdup(l);
  sprintf(l,"%.0f",r->Rdsr2);
  p->Rdsr2v = strdup(l);

  sprintf(l,"%.0f",r->Rmeas);
  p->Rmeasv = (r->Rmeas)? strdup(l) : strdup(NA);

  /* Rc, just N/A if set to zero */ 
  sprintf(l,"%g",r->Rc);
  p->Rcv = (!r->Rc) ? strdup(NA) : strdup(l);

  /* Gd, just N/A if set to zero */
  sprintf(l,"%.1f",r->Gid);
  p->Gidv = (!r->Gid) ? strdup(NA) : strdup(l);
  sprintf(l,"%.1f",r->Gd);
  p->Gdv = (!r->Gd) ? strdup(NA) : strdup(l);
  sprintf(l,"%.1f",r->Gmfgr);
  p->Gmfgrv = (!r->Gmfgr) ? strdup(NA) : strdup(l);
  p->Gdu = (!r->Gd) ? strdup("") : strdup(VMS);
  p->Gidu = (!r->Gid) ? strdup("") : strdup(VMS);
  p->Gmfgru = (!r->Gmfgr) ? strdup("") : strdup(VMS);
  p->Guidu = (!(strcmp(p->Rdeov,NA))) ? strdup("") : strdup(VMS);
  sprintf(l,"%g",r->Mmfgr);
  p->Mmfgrv = (!r->Mmfgr) ? strdup(NA) : strdup(l);
  sprintf(l,"%g",r->Fmfgr);
  p->Fmfgrv = (!r->Fmfgr) ? strdup(NA) : strdup(l);
  sprintf(l,"%g",r->Dcalc);
  p->Dcalcv = (!r->Dcalc) ? strdup(NA) : strdup(l);
  sprintf(l,"%g",r->Atten);
  p->Attenv = (!(strcmp(p->Rdeov,NA))) ? strdup(NA) : strdup(l);
  sprintf(l,"%.1f",r->Guid);
  p->Guidv = (!(strcmp(p->Rdeov,NA))) ? strdup(NA) : strdup(l);
  sprintf(l,"(%g, \\261 %s%g)",r->paz[0].r,(r->Did>1)?"":"i",r->paz[0].i);
  p->Poles = strdup(l);
  sprintf(l,"(%g, \\261 %s%g)",r->paz[2].r,(r->Did>1)?"":"i",r->paz[2].i);
  p->Zeroes = strdup(l);

  /* display string for the transfer instructions */
  /* this is also spit out for the response table value */
  sprintf(l,"%g",r->G);
  p->Gv = strdup(l);
}
