#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <memory.h>
#include "response.h"
#include "resp_info.h"
#include "parse.h"

char * type2str(int);
/*
static response_info_table rt;
*/
static RESPINFO rt;
RESPINFO *respinfo_head = NULL, *respinfo_tail = NULL;

// This first setup is mainly just for help purposes, giving
// descriptions of the different fields of the respinfo tables
#define TLD(a,b,c,d) {(char *) &(b)-(char *) &(a),(c),(d)}
static RESPINFOTABDESC resp_desc[] = {
  TLD(rt,rt.file,"The response recording file name",""),
  TLD(rt,rt.own,"Sensor owner",""),
  TLD(rt,rt.sit,"Site where cal was recorded",""),
  TLD(rt,rt.mod,"Sensor Model",""),
  TLD(rt,rt.ser,"Sensor Serial Number",""),
  TLD(rt,rt.opt,"Optional info",""),
  TLD(rt,rt.cmp,"Sensor component",""),
  TLD(rt,rt.pol,"Pulse polarity",""),
  TLD(rt,rt.Gd,"Damped sensitivity Calculated","V/m/s"),
  TLD(rt,rt.Guid,"Undamped sensitivity calculated from observed Gid","V/m/s"),
  TLD(rt,rt.Gmfgr,"Undamped sensitivity from manufacturer","V/m/s"),
  TLD(rt,rt.Gid,"Observed sensitivity from llnlid info and Iext","V/m/s"),
  TLD(rt,rt.Iext,"Current measured during calibration pulse","ma"),
  TLD(rt,rt.Rc,"Coil resistance (usu. from an accurate measurement)","ohms"),
  TLD(rt,rt.Rdeo,"Shunt resistance","ohms"),
  TLD(rt,rt.Rdsr1,"Resistance in series with shunt between shunt and coil","ohms"),
  TLD(rt,rt.Rdsr2,"Resistance in series with shunt between shunt and coil (if a pair is used)","ohms"),
  TLD(rt,rt.Mmfgr,"Weight of sensor mass","Kg"),
  TLD(rt,rt.Rmfgr,"Coil resistance from manufacturer","ohms"),
  TLD(rt,rt.Tmfgr,"Natural period of sensor from manufacturer","seconds"),
  TLD(rt,rt.Fmfgr,"Natural frequency of sensor from manufacturer (1/Tmfgr)","Hz"),
  TLD(rt,rt.Cmfgr,"Calibration constant of sensor from manufacturer",""),
  TLD(rt,rt.Dmfgr,"Open Circuit damping constant of sensor from manufacturer","% of critical"),
  TLD(rt,rt.Dcalc,"Damping constant of sensor (Calculated from Dmfgr)","% of critical"),
  TLD(rt,rt.Rmeas,"Resistance of sensor including damping array","ohms"),
  TLD(rt,rt.Rext,"Inline resistance used with calibration box to limit current","ohms"),
  TLD(rt,rt.Isig,"Current calulated as Iext * Atten (seems to be unused now)","ma"),
  TLD(rt,rt.Vext,"Voltage across sensor and damping array.  Used to calculate Iext if Iext doesn't exist.  Typically this is not as accurate.","volts"),
  TLD(rt,rt.Did,"Damping constant (from llnlid program)","% of critical"),
  TLD(rt,rt.GNid,"Dimensionless sensitivity constant (from llnlid program).  Used to calculate Gid","dimensionless constant"),
  TLD(rt,rt.Tid,"Natural Period (from llnlid program)","Hz"),
  TLD(rt,rt.Fid,"Natural frequency (from llnlid program 1/Tid)","seconds"),
};

#define TL(t,i,f,a,b) {(t),(i),(f),(char *) &(b)-(char *) &(a)}
static struct response_info_table resp_info[] = {
  TL("filename", P_STRING,  "%s",rt,rt.file),
  TL("file", P_STRING,  "%s",rt,rt.file),
  TL("owner", P_STRING,  "%s",rt,rt.own),
  TL("own", P_STRING,  "%s",rt,rt.own),
  TL("site", P_STRING,  "%s",rt,rt.sit),
  TL("sit", P_STRING,  "%s",rt,rt.sit),
  TL("model", P_STRING,  "%s",rt,rt.mod),
  TL("mod", P_STRING,  "%s", rt,rt.mod),
  TL("type", P_STRING,  "%s", rt,rt.mod),
  TL("senstype", P_STRING,  "%s",rt,rt.mod),
  TL("sensor", P_STRING,  "%s",rt,rt.mod),
  TL("sn", P_STRING,  "%s",rt,rt.ser),
  TL("ser", P_STRING,  "%s",rt,rt.ser),
  TL("serial", P_STRING,  "%s",rt,rt.ser),
  TL("serialnum", P_STRING,  "%s",rt,rt.ser),
  TL("s#", P_STRING,  "%s",rt,rt.ser),
  TL("sn#", P_STRING,  "%s",rt,rt.ser),
  TL("opt", P_STRING,  "%s",rt,rt.opt),
  TL("optdes", P_STRING,  "%s",rt,rt.opt),
  TL("option", P_STRING,  "%s",rt,rt.opt),
  TL("comp", P_STRING,  "%s",rt,rt.cmp),
  TL("cmp", P_STRING,  "%s",rt,rt.cmp),
  TL("c", P_STRING,  "%s",rt,rt.cmp),
  TL("pol", P_STRING,  "%s",rt,rt.pol),

  TL("Rc", P_FLOAT,  "%g",rt,rt.Rc),
  TL("Rcoil", P_FLOAT,  "%g", rt, rt.Rc),
  TL("Gd", P_FLOAT,  "%g", rt, rt.Gd),
  TL("cal", P_FLOAT,  "%.3f", rt, rt.Cmfgr),
  TL("calconst", P_FLOAT,  "%.3f", rt, rt.Cmfgr),
  TL("calconstant", P_FLOAT,  "%.3f", rt, rt.Cmfgr),
  TL("Cmfgr", P_FLOAT,  "%.3f", rt, rt.Cmfgr),
  TL("Gs", P_FLOAT,  "%g", rt, rt.Gd),
  TL("Gdamp", P_FLOAT,  "%g", rt, rt.Gd),
  TL("Rd", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rdeo", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rdpar", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rs", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rshunt", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rdamp", P_FLOAT,  "%g", rt, rt.Rdeo),
  TL("Rdsr", P_FLOAT,  "%g", rt, rt.Rdsr1),
  TL("Rdsr1", P_FLOAT,  "%g", rt, rt.Rdsr1),
  TL("Rdsr2", P_FLOAT,  "%g", rt, rt.Rdsr2),

  TL("Rmfgr", P_FLOAT,  "%g", rt, rt.Rmfgr),
  TL("Fmfgr", P_FLOAT,  "%.2f", rt, rt.Fmfgr),
  TL("F", P_FLOAT,  "%.2f",rt,rt.Fmfgr),
  TL("Tmfgr", P_FLOAT,  "%.2f", rt, rt.Tmfgr),
  TL("T", P_FLOAT,  "%.2f", rt, rt.Tmfgr),
  TL("Mmfgr", P_FLOAT,  "%.3f", rt, rt.Mmfgr),
  TL("Mass", P_FLOAT,  "%.3f", rt, rt.Mmfgr),
  TL("mass", P_FLOAT,  "%.3f", rt, rt.Mmfgr),
  TL("m", P_FLOAT,  "%.3f", rt, rt.Mmfgr),
  TL("Dmfgr", P_FLOAT,  "%.3f", rt, rt.Dmfgr),
  TL("Dopen", P_FLOAT,  "%.3f", rt, rt.Dmfgr),
  TL("Gmfgr", P_FLOAT,  "%.3f", rt, rt.Gmfgr),

  TL("Dcalc", P_FLOAT,  "%.3f", rt, rt.Dcalc),
  TL("Isig", P_FLOAT,  "%.4f", rt, rt.Isig),
  TL("Iext", P_FLOAT,  "%.4f", rt, rt.Iext),
  TL("Vext", P_FLOAT,  "%.4f", rt, rt.Vext),
  TL("Rext", P_FLOAT,  "%g", rt, rt.Rext),
  TL("Rmeas", P_FLOAT,  "%g", rt, rt.Rmeas),
  TL("Rcmeas", P_FLOAT,  "%g", rt, rt.Rmeas),

  /* the following are probably only for output */
  /*
    Gid, not in respinfo
    May need to have values included there, or move them into the struct
  */
  TL("Did", P_FLOAT,  "%g", rt, rt.Did),
  TL("Gid", P_FLOAT,  "%g", rt, rt.Gid),
  TL("GNid", P_FLOAT,  "%g", rt, rt.GNid),
  TL("Guid", P_FLOAT,  "%g", rt, rt.Guid),
  TL("Tid", P_FLOAT,  "%g", rt, rt.Tid),
  TL("Fid", P_FLOAT,  "%g", rt, rt.Fid),
};

static int tabdescsize = (sizeof(resp_desc)/sizeof(resp_desc[0]));
static int tabsize = (sizeof(resp_info)/sizeof(resp_info[0]));
static int infocount;
static struct input_order_table io_tab[100];
static int numflds = 0;
static RESPINFO *info_tab[INFO_TAB_MAX];
static char header[512];

////////////////////////////////////////////////////////////////////////////////
int print_aliases(){
  RESPINFOTAB *r;
  RESPINFOTABDESC *d;
  int i,j;
  int count = 0;

  for (j=0;j<tabdescsize;j++){
    d = &resp_desc[j];
    for (i=count=0;i<tabsize;i++){
      r = &resp_info[i];
      if( r->offset == d->offset) {
	if( ! count ) {
          fprintf(stdout,"%s",r->tag);
	}
	else fprintf(stdout,",%s",r->tag);
	count++;
      }
    }
    fprintf(stdout,":%s:%s\n",d->desc,d->units);
  }

  // should check the other direction and make sure we didnt miss any
  for (i=0;i<tabsize;i++){
    r = &resp_info[i];
    for (j=count=0;j<tabdescsize;j++){
      d = &resp_desc[j];
      if( r->offset == d->offset)  count++;
    }
    if(! count) fprintf(stdout,"Didn't find any entry for offset %d, %s\n",r->offset,r->tag);
  }
}
////////////////////////////////////////////////////////////////////////////////
int html_aliases(){
  RESPINFOTAB *r;
  RESPINFOTABDESC *d;
  int i,j, type;
  int count = 0;

  fprintf(stdout,"<table>\n");
  fprintf(stdout,"<tr>\n");
  fprintf(stdout,"<th>Aliases</th>\n");
  fprintf(stdout,"<th>Data Type</th>\n");
  fprintf(stdout,"<th>Description</th>\n");
  fprintf(stdout,"<th>Units</th>\n");
  fprintf(stdout,"</tr>\n");
  for (j=0;j<tabdescsize;j++){
    fprintf(stdout,"<tr>\n  <td>");
    d = &resp_desc[j];
    for (i=count=0;i<tabsize;i++){
      r = &resp_info[i];
      if( r->offset == d->offset) {
	type = r->type;
	if( ! count ) {
          fprintf(stdout,"%s",r->tag);
	}
	else fprintf(stdout,",%s",r->tag);
	count++;
      }
    }

    fprintf(stdout,"</td>\n  ");
    fprintf(stdout,"<td>%s</td>\n  ",type2str(type));
    fprintf(stdout,"<td>%s</td>\n  ",d->desc);
    fprintf(stdout,"<td>%s</td>\n  ",d->units);
    fprintf(stdout,"</tr>\n");
  }
  fprintf(stdout,"</table>\n");
}
////////////////////////////////////////////////////////////////////////////////
int listlen() {
  int i = 1;
  RESPINFO *r;

  for(r=respinfo_head;r;r=r->next) i++;

  return(i);
}
/*
  This routine is used to compare command line args 
  for respinfo program
*/
////////////////////////////////////////////////////////////////////////////////
int str_match_resp_info(char * str, struct input_order_table * ios) {
  register int i;
  struct response_info_table *r;

  for(i=0;i<tabsize;i++){
    r = &resp_info[i];
    /* ignore case for comparison */
    if(!(strcasecmp(str,r->tag))){
      ios->type = r->type;
      ios->offset = r->offset;
      ios->fmt    = r->fmt;
      return(1);
    }
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int parse_input_filename(RESPINFO * r)
{
  char *b, *e;
  int matched = 0;
  char **fnamev;
  int fnamec;

  /* 
    need to parse out file name for critical info components
    fields are period '.' separated.

    minimum should be:  date.site.owner.model.serialnum.comp.sac
    can be:  date.site.owner.model.serialnum.comp.[pol].[opt].sac


    optional fields are the polarity and then an optional designator
    (unused so far).

    return number of components found
  */

  if(!r || !r->file || !*(r->file)) return(0);

  fnamec = split(r->file,'.',&fnamev);

  /* if final component is "sac", get rid of it */
  if(!(strcmp("sac",fnamev[fnamec-1]))) fnamec--;

  /* do case checking */
  if(fnamec >= 6 && fnamec <= 8 ){
    r->dat = fnamev[0];
    r->sit = fnamev[1];
    r->own = fnamev[2];
    r->mod = fnamev[3];
    r->ser = fnamev[4];
    r->cmp = fnamev[5];
    if(fnamec == 7) r->pol = fnamev[6];
    if(fnamec == 8) r->opt = fnamev[7];
  }
  else {
    if(v){
      fprintf(stderr,"filename %s not in expected standard format: date.site.owner.mod.ser.cmp.[sac]\n",r->file);
    }
  }

  r->parsed = TRUE;
  return(fnamec);
}
////////////////////////////////////////////////////////////////////////////////
int fill_respinfo(RESPINFO * s) {
  /* do some basic calculations here to fill out struct */
  if(!s->Tmfgr && s->Fmfgr) s->Tmfgr = 1.0/s->Fmfgr;
  if(!s->Fmfgr && s->Tmfgr) s->Fmfgr = 1.0/s->Tmfgr;

  /* should I copy over id information here? */
  /* the calling routine does not attach the respinfo struct */
}
////////////////////////////////////////////////////////////////////////////////
RESPINFO * get_respinfo_head() {
  return(respinfo_head);
}
////////////////////////////////////////////////////////////////////////////////
int cmp_filename(RESPINFO * b, RESPINFO * t) {
  if(!b || !t) return(0);

  if(t->file){
    if(b->file) {
      if(!(strcmp(b->file,t->file))) return(1);
    }
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int cmp_owner_model_ser_cmp_pol(RESPINFO * b, RESPINFO * t) {
  if(!b || !t) return(0);

  if(!t->file && t->own && t->ser && t->mod && t->cmp && t->pol){
    if(b->mod && b->own && b->ser && b->cmp && b->pol){
      if(!(strcmp(b->mod,t->mod)) && 
	 !(strcmp(b->own,t->own)) &&
	 !(strcmp(b->ser,t->ser)) &&
	 !(strcmp(b->cmp,t->cmp)) &&
	 !(strcmp(b->pol,t->pol)) ) return(1);
    }
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int cmp_owner_model_ser_cmp(RESPINFO * b, RESPINFO * t)
{
  if(!b || !t) return(0);

  if(!t->file && t->own && t->ser && t->mod && t->cmp){
    if(b->mod && b->own && b->ser && b->cmp){
      if(!(strcmp(b->mod,t->mod)) && 
	 !(strcmp(b->own,t->own)) &&
	 !(strcmp(b->ser,t->ser)) &&
	 !(strcmp(b->cmp,t->cmp)) ) return(1);
    }
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int cmp_owner_model(RESPINFO * b, RESPINFO * t) {
  if(!b || !t) return(0);

  if(!t->file && t->own && !t->ser && t->mod){
    if(b->mod && b->own){
      if(!(strcmp(b->mod,t->mod)) && !(strcmp(b->own,t->own))) return(1);
    }
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int cmp_model(RESPINFO * b, RESPINFO * t) {
  if(!b || !t) return(0);

  if(!t->file && !t->own && !t->ser && t->mod){
    if(b->mod) if(!(strcmp(b->mod,t->mod))) return(1);
  }
  return(0);
}
////////////////////////////////////////////////////////////////////////////////
int print_critical(RESPINFO * b, RESPINFO * t) {
// I dont think that we are guaranteed to have any of these fields
  if(! t)  err("print_critical","t",1);
  if( t->file) fprintf(stderr,"file %s, ",t->file);
  if( t->own) fprintf(stderr,"own %s, ",t->own);
  if( t->mod) fprintf(stderr,"mod %s, ",t->mod);
  if( t->ser) fprintf(stderr,"ser %s, ",t->ser);
  if( t->cmp) fprintf(stderr,"cmp %s",t->cmp);
  //fprintf(stderr,"file %s, own %s, mod %s, ser %s, cmp %s\n",t->file,t->own,t->mod,t->ser,t->cmp);
}

////////////////////////////////////////////////////////////////////////////////
/* if pass NULL in, should respond with the number of pointers searched */
RESPINFO * match_filename(char * file)
{
  RESPINFO *r;

  if(! file || ! *file) return(NULL);

  for(r=respinfo_head;r;r=r->next){
    if(r->file && !(strcmp(file,r->file))) return(r);
  }
  return(NULL);
}
////////////////////////////////////////////////////////////////////////////////
#ifdef OLD
RESPINFO * match_respinfo_filename(RESPINFO * r) {
  register int i;
  RESPINFO *s;

  /* check initially to see if valid pointer passed in */
  if(!r) {
    for(s=respinfo_tail,i=0;s;s=s->prev,i++);
    if(v) fprintf(stderr,"match_respinfo_filename(): passed in NULL ptr: searched %d entries\n",i);

    return(NULL);
  }

  /* check for filename match */
  if(r->file){
    for(s=respinfo_tail,i=0;s;s=s->prev,i++){
      if(s->file){
        if(!(strcmp(s->file,r->file))) return(s);
      }
    }
  }

  /* check for match on model, serial num and component */
  if (r->mod && r->ser && r->cmp){
    for(s=respinfo_tail,i=0;s;s=s->prev,i++){
      if(s->mod && s->ser && s->cmp){
        if(!(strcmp(s->mod,r->mod)) &&
           !(strcmp(s->ser,r->ser)) &&
           !(strcmp(s->cmp,r->cmp))){
               return(s);
        }
      }
    }
  }

  if(v){
    fprintf(stderr,"match_respinfo_filename() failed: searched %d entries\n",i);
  }
  return(NULL);
}
#endif
////////////////////////////////////////////////////////////////////////////////
RESPINFO * new_respinfo(RESPINFO * respinfo) {
  RESPINFO *new;

  if(!(new = (RESPINFO *) calloc(1,sizeof(RESPINFO)))) 
	 syserr("allocating respinfo","",1);
  new->next = NULL;

  if(respinfo){
    respinfo->next = new;
    new->prev  = respinfo;
  }

  return(new);
}
////////////////////////////////////////////////////////////////////////////////
/* this is to build the info file */
int read_info_table(char * file) {
  int i,e,l;
  char sep;
  FILE *fi;
  char buf[512];
  char bu[512];
  int flds;
  int firsttime = TRUE;
  int merge     = FALSE;
  int merge_new = 0;
  int errcnt    = 0;
  int cf, com, cm;
  int cmpflg;
  RESPINFO test, *t = &test, *r;

  sep = '\t';

  i = 0;
  if(!(fi=fopen(file,"r"))) syserr("opening file",file,1);
  if(v) fprintf(stderr,"reading info from %s\n",file);

  /* this loops on the current, NEW input file being processed */
  for(l=1;fgets(buf,512,fi);l++){
    if(buf[0] == '#' || buf[0] == '\n') continue;

    if(firsttime){
      /* Check if linked list has already been initiallized */
      if(respinfo_head) merge = TRUE;

      /* save a copy of header line from info file */
      strcpy(header,buf);
      flds = process_header_str(buf,sep);
      if(flds <= 0){
        err("flds <= 0",NULL,0);
        return(0);
      }
      firsttime = FALSE;
      continue;
    }

    /* first case covers new, base entries */
    if(!merge){
      respinfo_tail = r = new_respinfo(respinfo_tail);
      if(!respinfo_head) respinfo_head = respinfo_tail;

      errcnt += load_fields(buf,sep,r);
    }
    /* this is all merging below */
    else {
      /* clear temp respinfo struct */
      memset(t,0x00,sizeof(RESPINFO));

      /* this loads the fields into the temp respinfo struct */
      errcnt += load_fields(buf,sep,t);

      /*
	May eventually want to keep some statistics here.
      */
      /* check temp struct against each existing info structs */
      for(e=0,r=respinfo_head;r;r=r->next){
	cmpflg = 0;

       /*
       if( cmp_filename(r,t) || 
           cmp_owner_model(r,t) ||
           cmp_model(r,t) ) {
        */
	/*
	cmpflg |= (cmp_filename(r,t) << 0);
	cmpflg |= (cmp_owner_model(r,t) << 1);
	cmpflg |= (cmp_model(r,t) << 2);
	*/
	cmpflg |= (cmp_filename(r,t)) << 0 ;
	cmpflg |= (cmp_owner_model_ser_cmp_pol(r,t)) << 1;
	cmpflg |= (cmp_owner_model_ser_cmp(r,t)) << 2;
	cmpflg |= (cmp_owner_model(r,t)) << 3;
	cmpflg |= (cmp_model(r,t)) << 4;

        
	if(cmpflg) {
	    if(v) fprintf(stderr,"cmpflg 0x%x\n",cmpflg);
	    if(v) print_critical(r,t);
             e++;
             errcnt += load_fields(buf,sep,r);
        }
	else {
	  /* this entry fails to match any of the base entries */
	} /* end of match failure else */
      }   /* end of for loop */
      if(v && e) fprintf(stderr,"line %2d of file %s matched %2d existing entries\n",l,file,e);
    }     /* end of merge else */

    /* insure filename parsing, this should happen in the first pass */
    if(r && !r->parsed && r->file) parse_input_filename(r);

    /* in merge mode, i no longer reflects the actual number merged */
    /* that number would now be i - merge_new */
    i++;
  }  /* end of while loop */
  fclose(fi);

  /* ouput statistics etc for this pass */
  if(errcnt){
    fprintf(stderr,"%d field loading errors counted while loading/merging file %s\n",errcnt,file);
  }

  if(merge_new && v) fprintf(stderr,"Allocated %d new respinfo structs in merge mode\n",merge_new);

  return((infocount=(i-merge_new)));
}

/* 
  resolve available fields and number them 
  assume single character field separator for now 
*/
////////////////////////////////////////////////////////////////////////////////
int process_header_str(char * str, char  sep) {
  register int i,j;
  char *c, *s;
  int num;
  int recognized = 0;
  static char recstr[512] = { "" };

  if(!sep) sep = '\t';

  /* initialize c to point to one byte before the actual dupped string */
  c=strdup(str);

  /* whack off the \n if it exists */
  if(*(s = (c+(strlen(c)-1))) == '\n') *s = NULL;
  i=0;
  do{
    s = c;
    if((c=index(s,sep))) *c++ = NULL;

    for(j=0;j<tabsize;j++){
      /*
      if(debug) fprintf(stderr,"process_header_str(): s :%s:,c :%s, resp_info[%d].tag :%s:\n",s,c,j,resp_info[j].tag);
      */

      /* ignore case in comparisons */
      if(!(strcasecmp(s,resp_info[j].tag))){
	strcat(recstr,s); strcat(recstr,", ");
	io_tab[i].type = resp_info[j].type;
	io_tab[i].offset = resp_info[j].offset;
	io_tab[i].field = resp_info[j].tag;
	recognized++;
	break;
      }
    }
    if(j>= tabsize){
      io_tab[i].type = 0;
      io_tab[i].offset = -1;
    }
    i++;
  } while (c);

  /* load up offsets and types into the tab struct */
  if(v) fprintf(stderr,"process_header_str(): numflds = %d, recognized = %d : %s\n",i, recognized,recstr);
  return( (numflds=i) /* number of fields */);
}
////////////////////////////////////////////////////////////////////////////////
/* 
  load_fields destructs the string str passed in as an arg
  loads only valid fields 

  modified to be less destructive on 10/12/94
  will now allocate memory as needed.  This will be more memory
  intensive, but should not really be excessive.

  use free at the end of the routine, should clean it up OK
*/
////////////////////////////////////////////////////////////////////////////////
int load_fields(char * str, char  sep, RESPINFO * respinfo) {
  register int i;
  char *s, *c;
  int errcnt = 0;

  if(!respinfo) return(NULL);

  c=strdup(str);
  /*
  c = str;
  */

  /* whack off the \n if it exists */
  /* the loop cycles on all fields in the line */
  if(*(s = (c+(strlen(c)-1))) == '\n') *s = NULL;
  i=0;
  do{
    s = c;
    if((c=index(s,sep))) *c++ = NULL;
    errcnt += str_fld_to_respinfo(s,i++,respinfo);
  } while(c);

  if(errcnt && v){
    fprintf(stderr,"%d field errs counted while loading fields to respinfo\n",errcnt);
  }
  free(c);
  return(errcnt);
}
////////////////////////////////////////////////////////////////////////////////
char *type2str(int type){
  switch( type ) {
    case P_FLOAT :
      return "float";
      break;
    case P_INT :
      return "int";
      break;
    case P_STRING :
      return "string";
      break;
    default:
      return "not defined";
      break;
  }
}
////////////////////////////////////////////////////////////////////////////////
int str_fld_to_respinfo(char * str, int  fld, RESPINFO * respinfo) {
  /* the p suffix indicates a pointer to the stored value */
  int i, *ip;
  float f, *fp;
  double d, *dp;
  char *s, **sp;
  char *h;
  //int atof();
  int offset;
  int errval = 0;

  if(!respinfo) return(0);

  /* what about ! str ?, could it happen */

  offset = io_tab[fld].offset;
  if(offset < 0) return(0);

  switch( io_tab[fld].type ) {
    case P_FLOAT :
      f = atof(str);
      fp = (float *) ((char *) respinfo+offset);
      if(!f && *fp ){
	fprintf(stderr,"WARNING: non-zero value %g replaced with zero value from string :%s: for field %s\n",*fp,str,io_tab[fld].field);
	errval++;
      }
      * (float *) ((char *) respinfo+offset) = f;
      //fprintf(stderr,"value %g from str %s being inserted into field #%d \n",f,str,fld);
      break;
    case P_INT :
      i = atoi(str);
      ip = (int *) ((char *) respinfo+offset);
      if(!i && *ip ){
	fprintf(stderr,"WARNING: non-zero value %d replaced with zero value from string :%s: for field %s\n",*ip,str,io_tab[fld].field);
	errval++;
      }
      * (int *) ((char *) respinfo+offset) = i;
      break;
    case P_STRING :
      /* had problems with this line actually with s */
      s = strdup(str);
      sp = (char **) ((char *) respinfo+offset);
      if(!*s && *sp ){
	fprintf(stderr,"WARNING: non-zero string %s replaced with empty string for field %s\n",*sp,io_tab[fld].field);
	errval++;
      }

      if(!*s) * (char **) ((char *) respinfo+offset) = NULL;
      else    * (char **) ((char *) respinfo+offset) = s;
      break;
  }
  return(errval);
}
////////////////////////////////////////////////////////////////////////////////
int process_all() {
  RESPINFO *r;

  for(r=get_respinfo_head();r;r=r->next) process_respinfo(r);
}
////////////////////////////////////////////////////////////////////////////////
int process_single(char * name) {
  process_respinfo(match_filename(name));
}
////////////////////////////////////////////////////////////////////////////////
void print_respinfo(RESPINFO *r){
  fprintf(stderr,"Tid=%g, Fid=%g, Did=%g, Gid=%g, GNid=%g\n",r->Tid,r->Fid,r->Did,r->Gid,r->GNid);
  fprintf(stderr,"Rc=%g, Gd=%g, G=%g, Rdeo=%g, Rdsf1=%g, Rdsr2=%g, Rmeas=%g, Rmfgr=%g\n",r->Rc,r->Gd,r->G,r->Rdeo,r->Rdsr1,r->Rdsr2, r->Rmeas, r->Rmfgr);
}
