/*
*/
#ifndef _resp_info_h
#define _resp_info_h
typedef struct response_info_table_desc {
  int offset;
  char *desc;
  char *units;
} RESPINFOTABDESC;
typedef struct response_info_table {
  char *tag;
  int   type;
  char *fmt;
  int offset;
}RESPINFOTAB;
typedef struct input_order_table {
  int type;
  int offset;
  char *field; /* field name */
  char *fmt;
}IOTAB;

#define INFO_TAB_MAX 1000

typedef struct complex_pair {
  float r,i;
} complex;

/* print string struct, attached as needed to RESPINFO */
typedef struct psstr {
  char *Rcv;
  char *Rdeov;
  char *Rdsr1v;
  char *Rdsr2v;
  char *Rmeasv;
  char *Gdv;
  char *Gdu;
  char *Gmfgrv;
  char *Gmfgru;
  char *Fmfgrv;
  char *Dcalcv;
  char *Mmfgrv;
  char *Gv;
  char *GNidv;
  char *Guidv;
  char *Guidu;
  char *Gidv;
  char *Gidu;
  char *Attenv;
  char *Poles;
  char *Zeroes;
} PSSTR;

/* this is used to read in the table produced by id.csh */
typedef struct response_info {
  /* observed/measured information */
  char *file;
  float Tid;       /* Obs Free Period from id */
  float Fid;       /* Obs Natural Frequency from id (1/Tid) */
  float Did;       /* Obs Damping from id */
  float Gid;       /* Obs Generator constant from id (calc using GNid and Isig) */
  float GNid;      /* Gain from id (dimensionless for now) */
  char *own;      /* Hopefully less than this */
  char *sit;      /* Testing site */
  char *mod;      /* Sensor model, can be many lengths */
  char *ser;      /* Sensor serial number */
  char *cmp;      /* component, should be single char */
  char *dat;      /* date string */
  char *pol;      /* polarity, should be single char */
  char *opt;      /* polarity, should be single char */
  int year;
  int month;
  int day;

  /* below are pieces of provided/input information */
  float Rc;       /* Coil resistance (measured) */
  float Gd;       /* Damped generator constant (v/m/s) */
  float G;        /* Damped generator constant used for calcs (v/m/s) */
  float Rdeo;     /* Damping resistance, parallel,     (measured) */
  float Rdsr1;    /* Damping resistance, first series, (measured) */
  float Rdsr2;    /* Damping resistance, second series, (measured) */
  float Rmeas;    /* Measured res. of entire sensor with damping, ohms */ 

  float Rmfgr;    /* Coil resistance from mfgr ohms */ 
  float Fmfgr;    /* Manufacturers frequency, reciprocal of Tmfgr */
  float Tmfgr;    /* Manufacturers period, reciprocal of Fmfgr */
  float Mmfgr;    /* Manufacturers mass specification Kg */
  float Dmfgr;    /* Mfgr speced damping, open circuit, percent of critical */
  float Gmfgr;    /* Mfgr Generator constant, open circuit  V/m/s */
  float Cmfgr;    /* Cal constant from mfgr, Newtons/A */

  float Dcalc;    /* Calculated damping, from mfgr specs and measured vals */
  float Isig;     /* Measured Current through signal coil milliamps? */ 
  float Iext;     /* Measured Current through entire sensor array, milliamps */ 
  float Vext;     /* Measured Voltage across entire sensor array, volts */ 
  float Rext;     /* Ext. resistance limiting Isig, may not be needed, ohms */

  /* the following vals will usually be supplied externally */
  float Guid;      /* Undamped Generator constant backed out from Gid (v/m/s) */

  float Atten;    /* Attenuation array, calculated from R arrays */
  int parsed;     /* set to true once the filename is parsed */
  complex paz[4]; /* poles and zeroes */
  PSSTR *psstr;
  struct response_info *next;
  struct response_info *prev;
  } RESPINFO;

//RESPINFO *get_id_tab_entry();
//RESPINFO *alloc_id_tab_entry();
//RESPINFO *get_respinfo_head();
//RESPINFO *match_respinfo_filename();
//RESPINFO *match_model_exclusive();
//RESPINFO *match_filename();
//PSSTR *alloc_psstr();



// function prototypes for file: resp_info.c
int listlen();
int str_match_resp_info(char * str, struct input_order_table * ios);
int parse_input_filename(RESPINFO * r);
int fill_respinfo(RESPINFO * s);
RESPINFO * get_respinfo_head();
int cmp_filename(RESPINFO * b, RESPINFO * t);
int cmp_owner_model_ser_cmp_pol(RESPINFO * b, RESPINFO * t);
int cmp_owner_model_ser_cmp(RESPINFO * b, RESPINFO * t);
int cmp_owner_model(RESPINFO * b, RESPINFO * t);
int cmp_model(RESPINFO * b, RESPINFO * t);
int print_critical(RESPINFO * b, RESPINFO * t);
RESPINFO * match_filename(char * file);
RESPINFO * match_respinfo_filename(RESPINFO * r);
RESPINFO * new_respinfo(RESPINFO * respinfo);
int read_info_table(char * file);
int process_header_str(char * str, char  sep);
int load_fields(char * str, char  sep, RESPINFO * respinfo);
int str_fld_to_respinfo(char * str, int  fld, RESPINFO * respinfo);
int process_all();
int process_single(char * name);
void print_respinfo(RESPINFO *);


#endif/*_resp_info_h*/
