#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "response.h"
#include "resp_info.h"
#include "resp_plot.h"
#include "resp_table.h"
#include "resp_calc.h"
#include "sac.h"
#include "protos.h"
#include "resp_ticks.h"
#define FO fprintf(fo
#define alog(a) ((a <= 0.0) ? 0.0 : log10(a))
  
int resp_plot(RESPINFO * r) {
  register int i;
  FILE *fps;
  float *fq, *vs, *pd, *gd;
  float bf = .011, ef = 99.0, nf = 1.05;
  RESPINFO *s;
  register float f;
  char ofname[128];
  char *c;
  int len;

  /* set local pointer */

  /* 
    dont have Gd yet,  may be using the calced value 

    But I should have Gd already ?!

    I think that this decision should be made in the id_calcs routine
    If there is no good G for charting, then skip this step entirely.
  */
  /*
  fprintf(stderr,"s->Gd %g, s->G %g\n",s->Gd,s->G);
  */
  s = r;
  if(s->Gd <= 0.0 && !s->G) return(-1);

  /* get number of increments */
  for(f=bf,i=0;f<=ef;f*=nf,i++);
  len = i;

  /* allocate pointers */
  if(!(fq = (float *) calloc(len,sizeof(float)))) syserr("calloc","fq",1);
  if(!(vs = (float *) calloc(len,sizeof(float)))) syserr("calloc","vs",1);
  if(!(pd = (float *) calloc(len,sizeof(float)))) syserr("calloc","pd",1);
  if(!(gd = (float *) calloc(len,sizeof(float)))) syserr("calloc","gd",1);

  for(i=0,f=bf;f<=ef;f*=nf,i++){
    fq[i] = f;
    vs[i] = vel_sensitivity(f,r->Fid,r->G,r->Did);
    pd[i] = phase_shift(f,r->Fid,r->G,r->Did);
    gd[i] = group_delay(f,r->Fid,r->G,r->Did);
  }

  /* lob off last portion of input name ".sac" */
  if(!r->file || !*r->file) return(-2);
  strcpy(ofname,r->file);
  c = strrchr(ofname,'.');
  *c = NULL;
  strcat(ofname,".ps");
  fprintf(stderr,"%s\n",ofname);
  if(!(fps=fopen(ofname,"w"))) syserr("opening file",ofname,1);

  ps_prolog(fps,dpi);
  ICSLogo(fps);
  if(gen_resp_plots(fps,r,fq,vs,pd,gd,len,color) < 0){
    err("general failure on get_resp_plots",r->file,0);
    fclose(fps);
    return(-3);
  }
  ps_finish(fps,logo);

  fclose(fps);
  return(0);
}
// get_sac_data_files(rt, sac, mod, tms)
// RESPINFO *rt;
// float **sac, **mod, **tms;
int
get_sac_data_files(RESPINFO * rt, float ** sac, float ** mod, float ** tms)
{
  static char sacname[128], modname[128];
  static struct sacHead sachdr, modhdr;
  register int i;
  static int sacsize = sizeof(struct sacHead);
  float tim;
  float *lsac, *lmod, *ltms;
  int len;

  /* copy sac name to modname and append .idmod */
  /* there are some extra steps here */
  strcpy(sacname,rt->file);
  strcpy(modname,sacname);
  strcat(modname,".idmod");

  /*
  fprintf(stderr,"sacfilename %s, modfilename %s\n",sacname, modname);
  */

  if( !(access(sacname,R_OK)) && !(access(modname,R_OK))){
    len = read_sac_file(modname,&modhdr,sac,NULL);
    len = read_sac_file(sacname,&sachdr,mod,len);

    if(modhdr.npts > sachdr.npts){
      err("model file npts > sac file npts",modname,0);
      goto errorlab ;
    }

    if(!(ltms = (float *) calloc(modhdr.npts, sizeof(float)))){
      syserr("allocating memory for time series",modname,0);
      goto errorlab ;
    }

    /* generate time series */
    for(i=0,tim = 0.00 ;i<modhdr.npts;i++, tim += modhdr.delta) ltms[i] = tim;

    *tms = ltms;
    return(modhdr.npts);
  }

  errorlab : 
    if(*sac) free(*sac);
    if(*mod) free(*mod);
    if(*tms) free(*tms);

    return(0);
}

// gen_resp_plots(fo,rt,fq,vs,pd,gd,len,color)
// FILE *fo;
// struct resp_tab *rt;
// float *fq, *vs, *pd, *gd;
// int len;
// int color;
int gen_resp_plots(FILE * fo, RESPINFO * rt, float * fq, float * vs, float * pd, float * gd, int  len, int  color)
{
  struct resp_plot *rp;
  float *sac, *mod, *tms;
  int datalen = 0;
  int rerr;
  char errstr[256];

  /* try to find the sac files */
  /*
    build filenames from table info
     .sac        for data file
     .sac.idmod  for model file

     open files
     read header
     allocate buffers (3, data, model, time_series)
     read data
     create time series
     setup plot 
     call ps_graph
  */

  datalen = get_sac_data_files(rt, &sac, &mod, &tms);

  init_params(datalen);
  set_line_params(fo,color);

  rp = &resp_plots[0];
  if((rerr = set_scale_params(rp,fq,vs,len,0.0,0.0)) < 0){
    sprintf(errstr,"failure %d on set_scale_params",rerr);
    err(errstr,"vs",0);
    return(-1);
  }
  ps_graph(fo,rp,fq,vs,len);

  rp = &resp_plots[1];
  if((rerr = set_scale_params(rp,fq,pd,len,0.0,0.0)) < 0){
    sprintf(errstr,"failure %d on set_scale_params",rerr);
    err(errstr,"pd",0);
    return(-1);
  }
  ps_graph(fo,rp,fq,pd,len);

  rp = &resp_plots[2];
  if((rerr = set_scale_params(rp,fq,gd,len,0.0,0.0)) < 0){
    sprintf(errstr,"failure %d on set_scale_params",rerr);
    err(errstr,"gd",0);
    return(-1);
  }
  ps_graph(fo,rp,fq,gd,len);

  if(datalen){
    rp = &resp_plots[3];

    set_scale_params(rp,tms,sac,datalen,0.0,0.03);
    ps_graph(fo,rp,tms,sac,datalen);

    resp_plots[4] = resp_plots[3];
    rp = &resp_plots[4];
    rp->labmode  = 0;
    rp->label   = strdup("idmodel");
    rp->legendlab = strdup("Model");
    ps_graph(fo,rp,tms,mod,datalen);

    if(sac) free(sac);
    if(mod) free(mod);
    if(tms) free(tms);
  }

  ps_labels(fo,rt);
  return(0);
}
// ps_ticks(fo,rp)
// FILE *fo;
// struct resp_plot *rp;
int
ps_ticks(FILE * fo, struct resp_plot * rp)
{
  register int l;
  char labstr[64];

  /* major x ticks */
  FO,"vsfont %% set small font for tick labeling\n");
  if(rp->xmode == PLOT_LOG){
    FO,"/XT { log %.8f sub %.8f mul I } def\n",rp->lxmin,rp->xsc);
  }
  else {
    FO,"/XT { %.8f sub %.8f mul I } def\n",rp->xmin,rp->xsc);
  }

  if(rp->ymode == PLOT_LOG){
    FO,"/YT { log %.8f sub %.8f mul I } def\n",rp->lymin,rp->ysc);
  }
  else{
    FO,"/YT { %.8f sub %.8f mul I } def\n",rp->ymin,rp->ysc);
  }

  for(l=0;l<rp->xtmajl;l++){
    sprintf(labstr,(rp->xscfmt)?rp->xscfmt:"%g",rp->xtmaj[l]);
    /*
    FO,"NP %.5f XT 0 M 0 %.3f I RL S\n",rp->xtmaj[l],rp->majtlen);
    */
    FO,"NP %g XT 0 M 0 %.3f I RL S\n",rp->xtmaj[l],rp->majtlen);
    FO,"NP %g XT 0 M 0 -.13 I RM (%s) SHC\n",rp->xtmaj[l],labstr);
  }
  for(l=0;l<rp->xtminl;l++){
    FO,"NP %g XT 0 M 0 %.3f I RL S\n",rp->xtmin[l],rp->mintlen);
  }

  /* major y ticks */
  for(l=0;l<rp->ytmajl;l++){
    sprintf(labstr,(rp->yscfmt)?rp->yscfmt:"%g",rp->ytmaj[l]);
    FO,"NP 0 %g YT M %.3f I 0 RL S\n",rp->ytmaj[l],rp->majtlen);
    FO,"NP 0 %g YT M -.1 I -.05 I RM (%s) SHR\n",rp->ytmaj[l],labstr);
  }
  /* minor y ticks */
  for(l=0;l<rp->ytminl;l++){
    FO,"NP 0 %g YT M %.3f I 0 RL S\n",rp->ytmin[l],rp->mintlen);
  }

  free(rp->xtmaj);
  free(rp->xtmin);
  free(rp->ytmaj);
  free(rp->ytmin);
}
// ps_graph(fo,rp,x,y,len)
// FILE *fo;
// struct resp_plot *rp;
// float *x, *y;
// int len;
int
ps_graph(FILE * fo, struct resp_plot * rp, float * x, float * y, int  len)
{
  register int i,j,l;
  register float k;
  float xint, yint;
  static int labcounter = 0;
  static int graphcounter = 0;

  float *xtmaj, *ytmaj, *xtmin, *ytmin;
  int xtminlen, xtmajlen, ytminlen, ytmajlen;

  int textwidth;
  char fmtstr[64];
  float xf, yf;

  FO,"%% beg ps_graph for plot labeled %s\n",rp->label);
  FO,"GS NP %.2f I %.2f I T 0 0 M\n",rp->x, rp->y);
  if(rp->labmode){
    FO,"0 %.2f I RL %.2f I 0 RL 0 %.2f I N RL CP S\n",rp->h,rp->w,rp->h);
    FO,"0 %.2f .05 add I M smfont (%s) SH\n",rp->h,rp->label);
    FO,"-.38 I %.2f I 2 div M smfont (%s) 90 RSHC\n",rp->h,rp->ysclab);
    FO,"NP %.2f 2 div I -.28 I M smfont (%s) SHC\n",rp->w,rp->xsclab);

    /* do ticks if labmode is on */
    ps_ticks(fo,rp);

    labcounter = 0;
  }
  else  labcounter++;


  /* set line params */
  FO,"%% beg data graph for plot labeled %s\n",rp->label);

  /* the mod operator is a nasty hack to allow multiple files */
  FO,"SLP%d\n",(graphcounter%NUM_PLOTS));
  if(rp->legendlab){
    FO,"GS NP 1.0 I %g .25 sub I .20 I %d mul sub M\n",rp->h,labcounter);
    FO,".3 I 0 RL GS S GR .1 I -.04 I RM vsfont (%s) SHL GR\n",rp->legendlab);
  }

  /* graph actual data */
  FO,"NP\n%g XT %g YT M\n",x[0],y[0]);
  for(i=1;i<len;i++) FO,"%g XT %g YT L %% %d\n",x[i],y[i],i);
  FO,"S\n");

  FO,"GR %% end ps_graph for plot labeled %s\n",rp->label);

  graphcounter++;
}
// set_line_params(fo,color)
// FILE *fo;
// int color;
int
set_line_params(FILE * fo, int  color)
{
  register int i;

  if(color){
    FO,"/SLP0 { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");
    FO,"/SLP1 { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");
    FO,"/SLP2 { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");
    FO,"/SLP3 { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");
    FO,"/SLP4 { 0 1 1 setrgbcolor [10 10] 0 setdash 1 setlinewidth } def\n");
    FO,"/SCP0 { 0 1 0 setrgbcolor } def\n");
    FO,"/SCP1 { 1 0 0 setrgbcolor } def\n");
    FO,"/ITMC { 0 0 0.5 setrgbcolor } def\n");
    FO,"/VALC { 0 0.8 0 setrgbcolor } def\n");
  }
  else {
    for(i=0;i<NUM_PLOTS;i++)
      FO,"/SLP%d { 0 setgray [] 0 setdash 1 setlinewidth } def\n",i);

    /* reset the model line to dashed */
    FO,"/SLP4 { 0 setgray [20 10] 0 setdash 1 setlinewidth } def\n");
    FO,"/SCP0 { 0 setgray } def\n");
    FO,"/SCP1 { 0 setgray } def\n");
    FO,"/ITMC { 0 setgray } def\n");
    FO,"/VALC { 0.2 setgray } def\n");
  }
  FO,"%% define a couple of popular alternatives\n");
  FO,"%% just redefine one of the numbers SLPs above\n");
  FO,"/SLPX { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");
  FO,"/SLPX { 0 1 1 setrgbcolor [10 10] 0 setdash 1 setlinewidth } def\n");
  FO,"/SLPX { 1 0 1 setrgbcolor [] 0 setdash 1 setlinewidth } def\n");

}
// init_params(have_data)
// int have_data;
int
init_params(int  have_data)
{
  register int i;
  struct resp_plot *rp;
  static float left, right, top, bottom = 2.50;
  static float height = 1.45 , gap = .6;

  /* set plot specific attributes */
  rp = &resp_plots[0];
  rp->label      = strdup("Velocity Sensitivity");
  rp->xmode      = PLOT_LOG;
  rp->ymode      = PLOT_LOG;
  rp->ymindiv    =    10;
  rp->ymin       =  0.00;
  rp->ymax       =  0.00;
  rp->ysclab     = strdup("V/m/sec");

  /*
  rp = &resp_plots[1];
  rp->label      = strdup("Phase Delay");
  rp->xmode      = PLOT_LOG;
  rp->ymode      = PLOT_LIN;
  rp->ymajtint   =  30.0;
  rp->ymintint   =  10.0;
  rp->ymindiv    =    0;
  rp->ymax       = 185.0;
  rp->ymin       = -5.0;
  rp->ysclab     = strdup("Degrees");
  */

  rp = &resp_plots[1];
  rp->label      = strdup("Phase Advance (steady state sinusoid)");
  rp->xmode      = PLOT_LOG;
  rp->ymode      = PLOT_LIN;
  rp->ymajtint   =     10.0;
  rp->ymindiv    =    10;
  rp->ymin       = -0.01;
  rp->ymax       =  0.00;
  rp->ysclab     = strdup("Seconds");

  rp = &resp_plots[2];
  rp->label      = strdup("Group Delay (wavelet)");
  rp->xmode      = PLOT_LOG;
  rp->ymode      = PLOT_LIN;
  rp->ymajtint   =     .10;
  rp->ymindiv    =    10;
  rp->ymin       = -0.01;
  rp->ymax       =  0.0000000;
  rp->ysclab     = strdup("Seconds");

  /* set common attributes for first 3 plots */
  for(i=0;i<3;i++){
    rp = &resp_plots[i];
    rp->x = (have_data) ? 1.2 : 1.5 ;
    rp->y = ((float) (2 - i) * (height + gap)) + bottom;
    rp->h = height;
    rp->w = (have_data) ? 3.60 : 5.40 ;
    rp->mintlen =     .040;
    rp->majtlen =     .060;
    rp->labmode = 1;
    rp->xmajdiv  = 0;
    rp->ymajdiv  = 0;
    rp->xsclab  = strdup("Frequency (Hz)");
    rp->xscfmt  = strdup("%g");
    rp->yscfmt  = strdup("%g");
    rp->legendlab = NULL;
  }

  if(have_data){
    rp = &resp_plots[3];
    rp->x = 5.70;
    rp->y = bottom; 
    rp->h = (height * 3) + (gap *2);
    rp->w = 2.30;
    rp->label      = strdup("Data/Model comparison");
    rp->ysclab     = strdup("Amplitude");
    rp->xsclab     = strdup("Time (secs)");
    rp->xmode      = PLOT_LIN;
    rp->ymode      = PLOT_LIN;
    rp->ymajtint   =  .100;
    rp->ymindiv    =  10;

    rp->ymajdiv    =  0;

    rp->xmajtint   =  0.5;
    rp->xmintint   =  0.10;
    /*
    rp->xmajtint   =  0.00;
    */
    rp->xmin       =  -0.01;
    rp->xmax       =  0.00;
    rp->ymin       =  0.00;
    rp->ymax       =  0.00;
    rp->labmode  = 1;
    rp->xscfmt  = strdup("%g");
    rp->yscfmt  = strdup("%.1f");
    rp->mintlen =     .040;
    rp->majtlen =     .060;
    rp->legendlab = strdup("Data");

  }
}
// get_plot_ticks(rp)
// struct resp_plot *rp;
int
get_plot_ticks(struct resp_plot * rp)
{

  /*
  fprintf(stderr,"starting get_plot_ticks\n");
  */

  if(rp->xmode == PLOT_LOG){
    rp->xtmajl = gen_log_ticks(rp->xmin,rp->xmax,1.0,&(rp->xtmaj),1);
    rp->xtminl = gen_log_ticks(rp->xmin,rp->xmax,0.0,&(rp->xtmin),1);
  }
  else{
    rp->xtmajl = gen_lin_ticks(rp->xmin,rp->xmax,rp->xmajtint,&(rp->xtmaj),1);
    rp->xtminl = gen_lin_ticks(rp->xmin,rp->xmax,rp->xmintint,&(rp->xtmin),1);
  }

  if(rp->ymode == PLOT_LOG){
    rp->ytmajl = gen_log_ticks(rp->ymin,rp->ymax,1.0,&(rp->ytmaj),1);
    rp->ytminl = gen_log_ticks(rp->ymin,rp->ymax,0.0,&(rp->ytmin),1);
  }
  else{
    rp->ytmajl = gen_lin_ticks(rp->ymin,rp->ymax,rp->ymajtint,&(rp->ytmaj),1);
    rp->ytminl = gen_lin_ticks(rp->ymin,rp->ymax,rp->ymintint,&(rp->ytmin),1);
  }

  /*
  fprintf(stderr,"get_plot_ticks x: min %g, max %g\n",rp->xmin,rp->xmax);
  fprintf(stderr,"get_plot_ticks y: min %g, max %g\n",rp->ymin,rp->ymax);
  fprintf(stderr,"get_plot_ticks y: tmaj[0] %g, tmaj[%d] %g\n",rp->ytmaj[0],rp->ytmajl-1,rp->ytmaj[rp->ytmajl-1]);
  fprintf(stderr,"get_plot_ticks y: tmin[0] %g, tmin[%d] %g\n",rp->ytmin[0],rp->ytminl-1,rp->ytmin[rp->ytminl-1]);
  */
}
// get_major_tick_interval(range, divisions, val)
// float range, *val;
// int divisions;
int
get_major_tick_interval(float  range, int  divisions, float * val)
{
  float inc;
  register int i;

  inc = range/(float) divisions;

    if(inc < 1.0){
      for(i=0;inc < 1.0;i++,inc *= 10.000);
      inc = (int) inc;
      for(;i>= 0;i--,inc /= 10.000);
    }
    else if(inc > 10.0){
      for(i=0;inc > 10.0;i++,inc /= 10.000);
      inc = (int) inc;
      for(;i>0;i--,inc *= 10.000);
    }
    else{
      inc = (int) inc;
    }
    *val = inc;
}
/*
////////////////////////////////////////////////////////////////////////////////
//  uses the tick calculation routine to produce the next set of ticks outside
//  of min max range (flag 0).
//
//  if the individual component is not preset to a val, then the appropriate 
//  tick val is used.
//
//  Used only if you desire to get range outside of data vals
////////////////////////////////////////////////////////////////////////////////
*/
int set_min_max_outside(RESPPLOT * rp, float * x, float * y, int  len) {
  register int i;
  float xmin,xmax,ymin,ymax;

  /* get min max info */
  min_max_flt(x,len,&xmin,&xmax);
  min_max_flt(y,len,&ymin,&ymax);

  if(v) fprintf(stderr," ----------------------------------------- \n");
  if(v) fprintf(stderr,"set_min_max_outside data x: min %g, max %g\n",xmin,xmax);
  if(v) fprintf(stderr,"set_min_max_outside data y: min %g, max %g\n",ymin,ymax);
  if(v) fprintf(stderr,"set_min_max_outside: majtint: x %g, y %g\n",rp->xmajtint,rp->ymajtint);

  if(rp->xmode == PLOT_LOG){
    rp->xtmajl = gen_log_ticks(xmin,xmax,1.0,&(rp->xtmaj),0);
  }
  else{
    rp->xtmajl = gen_lin_ticks(xmin,xmax,rp->xmajtint,&(rp->xtmaj),0);
  }

  if(rp->ymode == PLOT_LOG){
    rp->ytmajl = gen_log_ticks(ymin,ymax,1.0,&(rp->ytmaj),0);
  }
  else{
    rp->ytmajl = gen_lin_ticks(ymin,ymax,rp->ymajtint,&(rp->ytmaj),0);
  }

  if(!rp->ytmajl || !rp->xtmajl) return(-1);

  if(v) fprintf(stderr,"set_min_max_outside tick x: min %g, max %g\n",rp->xtmaj[0],rp->xtmaj[rp->xtmajl-1]);
  if(v) fprintf(stderr,"set_min_max_outside tick y: min %g, max %g\n",rp->ytmaj[0],rp->ytmaj[rp->ytmajl-1]);

  if(!rp->xmin) rp->xmin = rp->xtmaj[0];
  if(!rp->xmax) rp->xmax = rp->xtmaj[rp->xtmajl-1];
  if(!rp->ymin) rp->ymin = rp->ytmaj[0];
  if(!rp->ymax) rp->ymax = rp->ytmaj[rp->ytmajl-1];

  free(rp->xtmaj);
  free(rp->ytmaj);

  if(v) fprintf(stderr,"set_min_max_outside plot x: min %g, max %g\n",rp->xmin,rp->xmax);
  if(v) fprintf(stderr,"set_min_max_outside plot y: min %g, max %g\n",rp->ymin,rp->ymax);
  /*
  */
  return(0);
}
// perc_expand(min,max,perc)
// float *min, *max, perc;
int
perc_expand(float * min, float * max, float  perc)
{
  float expand;

  expand = ( *max - *min) * perc;
  *max += expand;
  *min -= expand;
}
// set_scale_params(rp,x,y,len,xperc,yperc)
// struct resp_plot *rp;
// float *x, *y;
// int len;
// float xperc, yperc;
int
set_scale_params(RESPPLOT * rp, float * x, float * y, int  len, float  xperc, float  yperc)
{
  float xmin,xmax,ymin,ymax;

  /* need to figure out how to deal with data whose general range is unknown */
  if(rp->xmajdiv){
    min_max_flt(x,len,&xmin,&xmax);
    if(xperc > 0.0) perc_expand(&xmin,&xmax,xperc);
    get_major_tick_interval((xmax-xmin),rp->xmajdiv,&(rp->xmajtint));
  }

  if(rp->ymajdiv){
    min_max_flt(y,len,&ymin,&ymax);
    if(yperc > 0.0) perc_expand(&ymin,&ymax,yperc);
    get_major_tick_interval((ymax-ymin),rp->ymajdiv,&(rp->ymajtint));
  }

    /* ticking info needs to be done before ticks are generated */
    rp->xmintint = (rp->xmintint > 0.00) ?  rp->xmintint : 1.0 ;
    rp->ymintint = (rp->ymintint > 0.00) ?  rp->ymintint : 1.0 ;

    rp->xmintint = (rp->xmindiv > 1) ?  rp->xmajtint/rp->xmindiv : rp->xmintint;
    rp->ymintint = (rp->ymindiv > 1) ?  rp->ymajtint/rp->ymindiv : rp->ymintint;

  if(xperc || yperc){
    min_max_flt(x,len,&xmin,&xmax);
    min_max_flt(y,len,&ymin,&ymax);
    if(xperc > 0.0) perc_expand(&xmin,&xmax,xperc);
    if(yperc > 0.0) perc_expand(&ymin,&ymax,yperc);
    if(!rp->xmin) rp->xmin = xmin;
    if(!rp->xmax) rp->xmax = xmax;
    if(!rp->ymin) rp->ymin = ymin;
    if(!rp->ymax) rp->ymax = ymax;
  }
  else {
    /* the set_min_max_outside sets the outside range to the next tick interval */
    /* that is its sole purpose */
    // original def, defies prototype
    //if(set_min_max_outside(rp,x,y,len,xperc,yperc) < 0) return(-1);
    if(set_min_max_outside(rp,x,y,len) < 0) return(-1);
  }
  get_plot_ticks(rp);

    /* get deltas for x and y */
    rp->dx = (rp->xmode == PLOT_LOG) ? 
	(alog(rp->xmax)-alog(rp->xmin)) : rp->xmax - rp->xmin ;
    rp->dy = (rp->ymode == PLOT_LOG) ? 
	(alog(rp->ymax)-alog(rp->ymin)) : rp->ymax - rp->ymin ;

    rp->xsc     = rp->w/rp->dx;
    rp->ysc     = rp->h/rp->dy;

    /* these are stored for ease of later scaling */
    rp->lxmin = (rp->xmode) ? alog(rp->xmin) : rp->xmin ;
    rp->lymin = (rp->ymode) ? alog(rp->ymin) : rp->ymin ;

  return(0);
}
