#ifndef _RESP_PLOT_H_
#define _RESP_PLOT_H_
#define NUM_PLOTS 5

enum plot_mode {
  PLOT_ERR,
  PLOT_NONE = 0,
  PLOT_LIN,
  PLOT_LOG,
  PLOT_LEN,
};

typedef struct resp_plot {
  char *label;
  char *xsclab;
  char *ysclab;
  char *legendlab;
  char *xscfmt, *yscfmt;
  int xmode, ymode;  /* plot_mode */
  int labmode;
  float xmajtint, ymajtint;  /* tick intervals  */
  float xmintint, ymintint;  /* tick intervals  */
  int xmindiv, ymindiv;      /* minor divisions */
  int xmajdiv, ymajdiv;      /* major divisions */
  float x, y;   /* in inches */
  float w, h;   /* in inches */
  float mintlen, majtlen;  /* in inches */

  /* these values provided */
  float xmin, xmax;
  float ymin, ymax;

  /* these values are calculated */
  float dx, dy;
  float xsc, ysc;
  float lxmin, lymin; /* for temp storage of log val */

  /* tick param storage */
  float *xtmaj, *xtmin, *ytmaj, *ytmin;
  int  xtmajl, xtminl, ytmajl, ytminl;
} RESPPLOT;

/* the +1 is a hack for the additional time series plot to the side */
static struct resp_plot resp_plots[NUM_PLOTS];

// function prototypes for file: resp_plot.c
int resp_plot(RESPINFO * r);
int get_sac_data_files(RESPINFO * rt, float ** sac, float ** mod, float ** tms);
int gen_resp_plots(FILE * fo, RESPINFO * rt, float * fq, float * vs, float * pd, float * gd, int  len, int  color);
int ps_ticks(FILE * fo, struct resp_plot * rp);
int ps_graph(FILE * fo, struct resp_plot * rp, float * x, float * y, int  len);
int set_line_params(FILE * fo, int  color);
int init_params(int  have_data);
int get_plot_ticks(struct resp_plot * rp);
int get_major_tick_interval(float, int, float *);
int set_min_max_outside(RESPPLOT * rp, float * x, float * y, int  len);
int perc_expand(float * min, float * max, float  perc);
int set_scale_params(RESPPLOT * rp, float * x, float * y, int  len, float  xperc, float  yperc);
#endif
