#include <stdio.h>
#include <math.h>
#include "resp_ticks.h"
#define alog(a) ((a <= 0.0) ? 0.0 : log10(a))

float myExp10(int num){
  int i;
  float retval = 1.0;
  for(i=0;i>num;i--) retval /= 10.0;
  for(i=0;i<num;i++) retval *= 10.0;
  return retval;
}
float get_lin_tick_min(float  min, float  interval) {
  int trunc;

  trunc = min/interval;
  if(trunc > 0) trunc--;

  return(trunc*interval);
}
float get_lin_tick_max(float  max, float  interval) {
  int trunc;

  trunc = max/interval;
  if(trunc < 0) trunc++;

  return(trunc*interval);
}

/* return the actual value of the max tick for a log scale */
float get_log_tick_max(float  max, float  interval) {
  int trunc;

  trunc = alog((max*1.000001));
  if(max < 1.0) trunc--;

  return(myExp10(trunc));
}

/* return the actual value of the min tick for a log scale */
float get_log_tick_min(float  min, float  interval) {
  int trunc;

  trunc = alog((min*.999999));
  if(min > 1.0) trunc++;

  return(myExp10(trunc));
}

/* return only the exponent (log) of the min */
float get_log_tick_min_log(float  min, float  interval) {
  int trunc;

  trunc = alog((min*.999999));
  if(min > 1.0) trunc++;

  return((float) trunc);
}
/* return only the exponent (log) of the max */
float get_log_tick_max_log(float  max, float  interval) {
  int trunc;

  trunc = alog((max*1.000001));
  if(max < 1.0) trunc--;

  return((float) trunc);
}

/*
  use interval here to resolve minor or major tick status
  if interval <= 0.0 then use minor, else use major.

  this is if min and max scale are known (not data vals)

  flag controls some aspects of the tick generation
    for now:
      if flag == 0, min and max are taken to be min and max data vals
      otherwise     min and max are taken to be min and max vals of graph scale

  normal use is to pass min and max data vals and major interval to routine
  with flag of 0
  next pass in min and max vals from major tick array with flag of 1
*/
// gen_log_ticks(imin,imax,interval,ticks,flag)
// float imin, imax, interval;
// float **ticks;
// int flag;
//int gen_log_ticks(float  imin, float  imax, float  interval, float ** ticks, int  flag) {
int gen_log_ticks(float  imin, float  imax, float  interval, float ** ticks, int  flag) {
  register int i,j;
  register float f,f2;
  int majlen, minlen;
  int lmin, lmax;
  float *maj, *min, *set;

  if(imin >= imax) return(0);

  if(imin <= 0.0 || imax <= 0.0) return(0);

  lmin = get_log_tick_min_log(imin,interval);
  lmax = get_log_tick_max_log(imax,interval);

  if(lmin >= lmax) return(0);

  if(!flag){
    lmin--;
    lmax++;
  }

  /*
  fprintf(stderr,"lmin %d, lmax %d\n",lmin,lmax);
  */

  /* major tick table */
  /* need to build major table regardless of which is output */
  majlen = (lmax-lmin)+1;
  if(!(maj=(float *) calloc(majlen,sizeof(float)))){
    fprintf(stderr,"unable to allocate requested memory %d\n",majlen);
    exit(1);
  }
  for(i=lmin,j=0;i<=lmax;i++,j++) maj[j] = myExp10(i);

  if(interval > 0.0) {
    *ticks = maj;
    return(majlen);
  }

  minlen = (majlen+1)*10;
  if(!(min=(float *) calloc(minlen,sizeof(float)))){
    fprintf(stderr,"unable to allocate requested memory %d\n",minlen);
    exit(1);
  }
  for(i=lmin-1, set = min;i<=lmax;i++){
    f = myExp10(i); 
    for(j=1;j<10;j++){
      f2 = f*j;
      if(f2 >= imin && f2 <= imax) *set++ = f2;
    }
  }
  *ticks = min;
  return(set - min);
}
/* 
  return len of array 

  return 0 on error
  This routine is used if you know the min and max of graph already

  flag controls some aspects of the tick generation
    for now:
      if flag == 0, min and max are taken to be min and max data vals, and
		    the range is increased by one interval on each side.

      otherwise     min and max are taken to be min and max vals of graph scale
		    and are not modified

  normal use is to pass min and max data vals and major interval to routine
  with flag of 0
  next pass in min and max vals from major tick array with flag of 1
*/
// gen_lin_ticks(imin,imax,interval,ticks,flag)
// float imin, imax, interval;
// float **ticks;
// int flag;
int
//gen_lin_ticks(float  imin, float  imax, float  interval, float ** ticks, int  flag)
gen_lin_ticks(float  imin, float  imax, float  interval, float ** ticks, int  flag)
{
  register int i;
  register float f;
  int len;
  float min, max;
  float *t, *set;

  if(imin >= imax) return(0);
  if(interval <= 0.0) return(0);

  min = get_lin_tick_min(imin,interval);
  max = get_lin_tick_max(imax,interval);

  if(! flag){
    min -= interval;
    max += interval;
  }

  /* the +interval*.1 is too make sure that the last tick isnt lobbed off */
  len = (((max - min)+(interval *.2))/interval)+1;
  if(!(t=(float *) calloc(len,sizeof(float)))){
    fprintf(stderr,"unable to allocate requested memory %d\n",len);
    exit(1);
  }

  for(i=0,f=min,set=t;i<len;f+=interval,set++,i++) *set = f;

  *ticks = t;
  
  return(len);
}
