#define _MAIN_
#include <stdio.h>
#include <string.h>
#include "response.h"
#include "response_release.h"
#include "sac.h"
#include "math.h"
#define USAGE "respest [-v] [-h[elp]] [-single] [-silent] [file ...]"
#define FALSE 0
#define TRUE  1

int usage(char *, int);
int main(int, char **);
int process_file(char *);
int cut_response(char *, struct sacHead *, float *, float *);
int level_crossings(float *, int, float);

  int help = FALSE;
  int maxlen;
  char sep = '\t';
  int Single = FALSE;
  int cut    = FALSE;
  float pre = .25, post = 1.0;
  float durfac = 1.45;
  float perfac = 3.2;

////////////////////////////////////////////////////////////////////////////////
int usage(char * prog, int  fatal) {
  fprintf(stderr,"%s %s\n",prog,VERSION);
  fprintf(stderr,"  usage: %s\n",USAGE);
  if(fatal) exit(0);
}
////////////////////////////////////////////////////////////////////////////////
int main(int  argc, char ** argv) {
  register int i;
  int optind;
  char *arg;
  char *progname;
  int silent = FALSE;

  v = FALSE;
  progname = argv[0];

  /* parse args */
  for(optind=1;optind < argc && argv[optind][0] == '-';optind++){
    arg = argv[optind]+1;
    if      (!(strcmp(arg,"v")))   v=TRUE;
    else if (!(strcmp(arg,"h")) || !(strcmp(arg,"help"))) help=TRUE;
    else if (!(strcmp(arg,"single"))) Single=TRUE;
    else if (!(strcmp(arg,"silent"))) silent=TRUE;
    else if (!(strcmp(arg,"cut")))    cut=TRUE;
    else if (!(strcmp(arg,"pre"))){
      if((++optind) >= argc ) usage(progname,1);
      else  pre = atof(argv[optind]);
    }
    else if (!(strcmp(arg,"post"))){
      if((++optind) >= argc ) usage(progname,1);
      else  post = atof(argv[optind]);
    }
  }
  argc -= optind;
  argv += optind;

  if(!argc) usage(progname,1);

  if(silent) v=FALSE;
  else       fprintf(stderr,"%s %s\n",progname,VERSION);

  if(Single) argc = 1;

  /* determine max line len */
  for(i=0;i<argc;i++){
    process_file(argv[i]);
  }
}
////////////////////////////////////////////////////////////////////////////////
int process_file(char * file) {
  struct sacHead sac;
  float *data;
  int len, idur;
  float min, max;
  float avg, base, mean_flt();
  float d1,d2,h1,h2;  /* deltas and height */
  float Dest;
  float Test;
  float pol;
  float width;
  float trig_thresh;
  float *f, *nf;
  float duration;
  double lnr;
  register int i;
  float roundto = 0.5;

  fprintf(stderr,"processing file %s\n",file);
  if((len = read_sac_file(file,&sac,&data,NULL))<=0){
    fprintf(stderr,"No data read in from %s\n",file);
    return(0);
  }
  min_max_flt(data,len,&min,&max);
  base = mean_flt(data,(len>10)?10:len);
  avg = mean_flt(data,len);

/*
  fprintf(stderr,"min %g, max %g, base %g, avg %g\n",min,max,base,avg);
*/

  /* find the level that is halfway (or some other percentage) */
  /* of the highest level */
  d1 = max - base;
  d2 = base - min;

  /* min cannot be greater than base, but can be close at critical damping */ 
  /* this case should be impossible, but check anyway */
  if(d1 < 0.0 || d2 < 0.0){
    fprintf(stderr,"Bad delta: d1 %g, d2 %g\n",d1,d2);
    return(0);
  }

  /* get largest of the two deltas, that is the major peak amplitude */
  h1 = (d1 > d2)?d1:d2;
  pol = (d1 > d2)? 1.0 : -1.0;
  h2 = (d1 > d2)?d2:d1;

  /* find the first break, h1 is the larger of the swings */
  /* base + (h1 * pol) */
  trig_thresh = base + (h1 * pol * .1);  
  for(i=0,f=data,nf=data+1;i<(len-1);i++,f++,nf++){
    if(*f >= trig_thresh && *nf < trig_thresh) break;
    if(*f <= trig_thresh && *nf > trig_thresh) break;
  }
  
  if(cut) cut_response(file,&sac,data,f);

  /* find the number of samples between the zero crossings */
  /* the level will control it somewhat */
  width = (float) level_crossings(data,len,(base+(pol*(h1/2.0))))*sac.delta;

  /* can get a rough estimate of damping factor */
  /* this value cannot mathematically exceed 1.0, but should be accurate */
  /* for cases with zero crossings */
  lnr  = log(h1/h2);

  /* do estimate calculations */
  Dest = lnr/(sqrt((M_PI * M_PI) + (lnr * lnr)));
  Test = width*perfac;

  /* calculate a duration and try to round it to the nearest half second */
  /* truncate down to nearest .25 seconds */
  duration = Test/Dest*durfac;
  idur = duration/roundto;
  duration = (float) idur*roundto;
  if(duration < roundto) duration = roundto;

  if(v) fprintf(stdout,"estimated damping %g\n",Dest);
  if(v) fprintf(stdout,"crossing width %g\n",width);
  if(v) fprintf(stdout,"estimated period %g\n",Test);
  if(v) fprintf(stdout,"estimated duration %g\n",duration);
}
////////////////////////////////////////////////////////////////////////////////
int cut_response(char * file, struct sacHead * sac, float * data, float * f) {
  int pre_s, post_s;
  int begs, ends;
  int bo; /* beginning offset */
  int b,e;

  fprintf(stderr,"trigger threshold crossed at sample %d\n",f - data);

  begs = (int) (f - data) - (int) (pre/sac->delta);
  if(begs < 0) begs = 0;

  ends = (int) (f - data) + (int) (post/sac->delta);
  if( ends > sac->npts ) ends = sac->npts;

/*
  fprintf(stderr,"f 0x%x %d, data 0x%x %d, diff %d\n",f,f,data,data,f-data);
  fprintf(stderr,"b %d, e %d\n",b,e);
  fprintf(stderr,"si %.4f, pre %.2f, post %.2f\n",sac->delta,pre,post);
*/
  fprintf(stderr,"samples for cut: beg %d, end %d, pre_s %d, post_s %d\n",begs,ends,pre_s,post_s);

  if(ends <= begs){
    fprintf(stderr,"error on beg and end samples ... trace not cut\n");
    return(0);
  }

  /* these timing corrections should be OK, I do not correct all the way
     to year though, only to the day */

  sac->npts = ends-begs;
  sac->e = sac->npts * sac->delta;
  sac->nzmsec += begs * sac->delta * 1000.0;
  while (sac->nzmsec >= 1000 ){
    sac->nzmsec -= 1000;
    sac->nzsec ++;
  }
  while(sac->nzsec >= 60){
    sac->nzsec -= 60;
    sac->nzmin ++;
  }
  while(sac->nzmin >= 60){
    sac->nzmin -= 60;
    sac->nzhour ++;
  }
  while(sac->nzhour >= 24){
    sac->nzhour -= 24;
    sac->nzday ++;
  }

  write_sac_file(file,sac,data+begs,sac->npts);
}
////////////////////////////////////////////////////////////////////////////////
/* 
  return the number of samples between first two transitions.
  or zero if never does a first or second transition

  changed it to between first and last transitions.
  tried, but undid this.
*/
int level_crossings(float * buf, int  len, float  level) {
  register int i;
  int fi = 0, li = 0;
  float *c, *l;

  for(i=1,c = buf-1, l=buf;i<len;i++,l++,c++){
    /* decide whether sample is above or below line */
    /* an equal would also indicate a transition */
    if((*l <= level && *c > level) || (*l >= level && *c < level)){
      if(!fi) fi = i;
      else       return(i-fi);
    }
  }
  return(0);
}
