#define _MAIN_
#include <stdio.h>
#include <string.h>
#include "response_release.h"
#include "response.h"
#define FALSE 0
#define TRUE  1
#define U1 "[-h[elp] [type]] [-v] [-respinfo] [-out outfile] [-cal] [-color] [-logo] [-tablist info_table_list_file] [-info info_table] [-dpi res] [file ...]"
#define U2 "[-h[elp]] [-v] [-out outfile] [-cal] [-single] [-silent] [-it id_table] [-info info_table] [-id_field] [file ...]"

int help(char * name, char * which) {
  usage(name,0);

  if( !strcmp(which,"aliases_html")){
    html_aliases();
  }
  else if(!strcmp(which, "aliases_text")){
    print_aliases();
  }
  else {
    help_response();
  }
  exit(1);
}
int help_response() {
   fprintf(stderr,"Allowed types for help are: aliases_html aliases_text\n");
  //print_aliases();
  //fprintf(stderr,"No help implemented yet\n");
}
int usage(char * name, int  fatal) {
  if(respinfo) fprintf(stderr,"  usage: %s %s\n",name, U2);
  else         fprintf(stderr,"  usage: %s %s\n",name, U1);
  if(fatal) exit(1);
}
int main(int  argc, char ** argv) {
  register int i;
  char *arg, *progname;
  int len;
  static char *infofile[5] = { NULL, NULL, NULL, NULL, NULL };
  FILE *fit, *ftablist;
  float atof();
  int tabcount;
  int info;
  int iind;
  int ic = 0;
  char *tail;
  char *tablist;
  char buf[128];
  int listcount = 0;

  /* initialize extern global vars */
  v = FALSE;
  color = FALSE;
  logo = FALSE;
  dpi = 288;
  cal = FALSE;
  respinfo = FALSE;
  maxlen  = 0;
  debug = FALSE;
  fout = NULL;
  comment = FALSE;

  progname = argv[0];

  /* see which name the program is being called by */
  if(!(tail = strrchr(progname,'/'))) tail = progname;

  /* if not called by respinfo, assume response */
  if(!(strcmp(tail,"respinfo"))) respinfo = TRUE;

  if(argc <= 1) {
    fprintf(stderr,"%s %s\n",progname,VERSION);
    usage(progname,1);
  }

  /* parse command line args */
  for(i=1;i<argc && argv[i][0] == '-';i++){
    arg = argv[i]+1;
    if(!(strcmp(arg,"h")) || !(strcmp(arg,"help"))){
      if((++i) >= argc) help(progname,"general");
      else{
	help(progname,argv[i]);
      }
      //usage(progname,1);
    }
    else if (!(strcmp(arg,"single")))    Single=TRUE;
    else if (!(strcmp(arg,"silent")))    silent=TRUE;
    else if(!(strcmp(arg,"v")))               v=TRUE;
    else if(!(strcmp(arg,"debug")))       debug=TRUE;
    else if(!(strcmp(arg,"cal")))           cal=TRUE;
    else if(!(strcmp(arg,"color")))       color=TRUE;
    else if(!(strcmp(arg,"logo")))         logo=TRUE;
    else if(!(strcmp(arg,"respinfo"))) respinfo=TRUE;
    else if(!(strcmp(arg,"comment")))   comment=TRUE;
    else if(!(strcmp(arg,"out"))){
      if((++i) >= argc) usage(progname,1);
      if(!(fout=fopen(argv[i],"a"))) syserr("opening file",argv[i],1);
    }
    else if(!(strcmp(arg,"dpi"))){
      if((++i) >= argc) usage(progname,1);
      dpi = atoi(argv[i]);
    }
    else if(!(strcmp(arg,"tablist"))){
      if((++i) >= argc) usage(progname,1);
      tablist = argv[i];
    }
    else if(!(strcmp(arg,"info"))){
      if((++i) >= argc) usage(progname,1);
      infofile[ic] = argv[i];
      if((++ic) > 5) ic--;
    }
    else{
      if(respinfo) load_resp_info_arg(arg);
      else         usage(progname,1);
    }
  }
  argc -= i;
  argv += i;

  if(!fout) fout=stdout;

  if(silent) v = FALSE;
  else       fprintf(stderr,"%s %s\n",progname,VERSION);

  if(Single) argc = 1;

  /* 
    read and merge info tables
    v3.0 no longer makes a distinction between the
    two types of input tables.  All tables are in the same format.
    So now there is not input table that needs to be run before information 
    tables.  Want to create a linked list of info that corresponds to
    individual input filenames from the tables.
  */
  if(tablist) {
    if(!(ftablist=fopen(tablist,"r"))) syserr("opening file",tablist,1);
    for(i=0;fscanf(ftablist,"%s",buf)>0;i++){
       info = read_info_table(buf);
       if(v) fprintf(stderr,"%s %d entries from %s\n\n",
		     (i)?"merged":"read",info,buf);
    }
    fclose(ftablist);
  }
  for(i=0;i<ic;i++){
    if(infofile[i] && *infofile[i]){
       info = read_info_table(infofile[i]);
       if(v) fprintf(stderr,"%s %d entries from %s\n\n",
		     (i)?"merged":"read",info,infofile[i]);
    }
  }

  if(respinfo && !silent && !Single) output_header_str();

  listcount = listlen();

  /* if no args on command line, do all entries in table */
  /* else match the arg */
  if(argc > 0){
    for(i=0;i<argc;i++){
      process_single(argv[i]);
    }
  }
  else {
    process_all();
  }
  exit(0);
}
