/*
  Created by Aaron J. Martin on 08/02/93

  program to calculate the 3 curves used for the sensor response plots.

  Modified on 08/10/93  1.01
    Added some new fields to the input table file and respective structs etc.
    This allows us to read in the appropriate fields.

  Modified on 08/16/93  1.02
    Modified the id.csh stuff.  Since table input is expected now, I use table
    info as input values passed to id.

  Modified on 08/17/93  1.02
    Added in some mods to response.  Am going to add a new field to the 
    input data file name: site.  This is will be most beneficial for field
    testing and calibration.  

    Added in a fourth plot.  The data and model fit.  For now, if the data
    is available, it is plotted.  May make it a command line option or a
    command line disable later.

    Found a bug in the line tick gen routine.  The routine was not filling
    the buffer all the way.  Left the final value as a zeroed data point.

    Want to modify the organization of the tick calcing.  It is ugly and 
    confusing right now.  Also all plots are treated the same.  I want to 
    hardwire the fourth plot to use the data xmin and xmax, not tick xmin and
    xmax like the other plots.

    Also modified the 2nd plot.  Changed it from a phase delay to a phase
    shift.  The formula is not correct.  I need to have Pete Rodgers look
    at it.

    Significantly modified the id.csh routine that builds the response table.

  Modified on 08/22/93  1.03 by Aaron J. Martin
    Added in some line params (setdash, setgray) prior to actually graphing
    the data.  This will allow easy modification to color and/or dashed lines.
    Also added in some comments that use the graph label to bracket items.
    Expanded the labmode to include the box drawing etc...

    Added in a legendlabel var in the resp_plot struct.

  Modified on 08/28/93  1.04 by Aaron J. Martin
    Added in some empty shows towards bottom for possible figure captions or 
    comments.  Define SCP0 for those strings.

  Modified on 08/30/93  1.05 by Aaron J. Martin
    Modified formatting of the Rd value.  If >= 2Mohms, then just
    print open.

  Modified on 10/28/93  1.06 by Aaron J. Martin
    Added in a check for argc <= 1, print usage line in that case.
    Need to check in to logo situation.  Seemed to print even without
    the -logo arg.

  Modified on 12/28/93  1.07 by Aaron J. Martin
    Changed the table scan routine to correspond to the new namemap format.
    Site now comes before owner.  It makes more sense that way.

  Modified on 12/29/93  2.00 by Aaron J. Martin
    Changed the way input is handled.  Response now does a lot more of the 
    required work.  The hardest part will be to parse out the info table
    values and figure out which values to use and which to calculate.

    Now I read in only the basic input from the id.csh produced table.
    The format there should now be only: filename To Do Gn.  These are
    the values that id produces.  Id will in turn use some of the info
    table stuff being built to get some of its desired values.

    Also, response now expects that there will be filename args on the 
    command line.  Response will take these names and try to find the 
    appropriate entry in the input table.  If not found, response will
    complain and skip to the next command line arg.

    Also added a new prog that will parallel response.  respinfo allows 
    the user to pull fields from a free format info file.  Uses the same
    routines as response for field searching.  Operates in a manner 
    similar to segyhead.

  Modified on 12/30/93  2.01 by Aaron J. Martin
    Modified the sac processing routine.  Made a routine to read the 
    single sac file.  Broke this code out into a separate source file
    (sac.c) and added it to libucsb (release 2.06?).  May use this
    in a response estimator program.

  Modified on 01/01/94  2.02 by Aaron J. Martin
    Found and fixed several bugs.  An obscure one concerning s->Rdeo
    was found.  If there were multiple entries for a sensor in -it table
    and table was used as input the second pass on the same sensor would
    use the previously modified value of s->Rdeo (100000000 ohms) and 
    produce the string "Open" instead of "N/A".

    Also modified some plotting stuff.  Waveform scaling was all messed up.
    Modified the way the min and max were processed and ticks calculated.
    The code is pretty hacked up.  There are some sound ideas, but the 
    execution can be confusing.  Anyway, the first 3 plots have not been 
    modified, but the waveform one is now scaled differently.

    Completely changed the format of the display information at the top 
    of the page.  Also broke that code out into a separate file.  
    Calculations and display decisions are all made in id_calcs.c now.
    This was broken out of the display code to avoid confusion.  There is
    now an additional pointer to a struct that contains the display strings
    produced in id_calcs.

  Modified on 01/07/94  2.03 by Aaron J. Martin
    Merged respinfo and response.  There ended up being far to many lines 
    of parallel code.  It made sense to merge the two.

  Modified on 01/09/94  2.04 by Aaron J. Martin
    Modified the read_info_table() code.  In merge mode, if no match is found,
    a new struct is now allocated and the values set.  The routine still
    returns the number merged.  In verbose mode, the routine now prints out
    the number of new structs allocated if any are.  This addition of code
    also should make the error checking more stable and consistent.  The 
    local sspec pointer should never be able to be NULL.

  Modified on 01/11/94  2.05 by Aaron J. Martin
    Added a new entry to sspec struct, Guid.  This will be the undamped Gen
    const backed out from Gid.  I feel that this value is redundant, but 
    Pete feels that it's important to have.  Now the two lines (last from mfgr
    and first from id) match item per item across the board.  The GNid value
    is moved to the next line down and the plots have been made smaller.
    In addition a pserr string has been added to occupy the space below the 
    id measurements.  This string will contain error information from other
    places in the program.  Currently it is only used for bad GNid information.

  Modified on 01/27/94  2.06 by Aaron J. Martin
    Added in some more error condition checking.  Was having some problems.
    Needed to track them down.

  Modified on 01/29/94  2.07 by Aaron J. Martin
    Modified some of the labeling stuff to be more consistent.

  Modified on 02/21/94  2.08 by Aaron J. Martin
    Worked on some badly explained errors.  Found one bug that tells 
    the user that only 1 field is parsed on the filename when all but
    one is parsed correctly.  No obvious reason there.

  Modified on 02/22/94  2.09 by Aaron J. Martin
    Added in some error checking on loading fields.  Check if a zero value
    is replacing a non-zero value.  If so print out a warning message.
    Also added Vext field value.  Vext and Rmeas can be used to calculate
    Iext if it is zero.  Should be all in place.

    Didnt get to track down the filename parsing problem encountered
    the other day (missing model field resulted in confusing error message).
    Hope to work on that problem soon.  Have not had a chance to really
    check some of the new additions.

  Modified on 03/09/94  2.10 by Aaron J. Martin
    Pete Rodgers noticed that the vel_sensitivity plot did not have any
    of the expected overshoot.  When we looked at the formula, Pete 
    noticed that the Do term in the denominator was not squared.  We
    dont know whether the term was miswritten initially before I coded it
    or whether it was a coding mistake.  Anyway this should change the 
    complexion of the plot a bit.  Not a major problem.

  Modified on 05/12/94  3.00 by Aaron J. Martin
    Substantial change from earlier version.  Need to change the organization
    of the input files.  Want to try first to match entries by filename, 
    secondly by model, sn and comp.  Will not rely on the chosen naming
    convention.  Instead use that as the most flexible case.  But allow
    other means of matching.  Want to have only one type of input format.
    So need to alter id.csh output to include field separator.  Maybe 
    try to make the field checker a little more flexible.  Maybe.

  Modified on 05/13/94  3.01 by Aaron J. Martin
    Cleaned up some code.  Got rid of the src file resp_input.c.  
    Code was not needed.  Merged the include files for resp_input
    and resp_info.  Only needed the one.  Fixed the makefile.
    The linking section was broken and not really in the most
    appropriate place.  Noticed that the v/m/s units label is
    not making it to the final output.  Need to figure out why.

  Modified on 09/20/94  3.01 by Aaron J. Martin
    Changed wording of one label "Shift" changed to "Advance".

  Modified on 10/12/94  3.02 by Aaron J. Martin
    Added and fixed code to make the merging more streamlined.  Hope
    it helps instead of hinders.  Added the code to try to allow 
    generic sensor model values.

  Modified on 10/25/94  3.03 by Aaron J. Martin
    Fixed load_ps_err_str().  The error value was never cleared 
    after an error resulting in good plots having error messages
    if a previous entry was bad in the same run.

  Modified on 11/17/94  3.04 by Aaron J. Martin
    Fix minor display bug in filename parsing routine.  Also add in more 
    info in that same routine.  Found a bogus line in the scecinfo.txt 
    file that was messing everything up.  It was a single entry in the 
    model column.  It was trying to replace all the valid fields with the
    zero attributes in the row (all fields!).  Also got rid of the creation
    section in the case where there are no matches.  The initial info table
    is the the master table, new new entries should be created, just 
    merged.

  Modified on 12/26/94  3.05 by Aaron J. Martin
    Added in a polarity field designator.  This was mostly in before.

  Modified on 03/17/95  3.06 by Aaron J. Martin
    Started adding in poles and zeroes calculations.  Need to figure 
    out where to put them now.  Will also need to format the complex
    numbers (1.xxxxx,1.xxxxxxi).

    Also added in an output file option -out.

  Modified on 04/19/95  3.07 by Aaron J. Martin
    Poles and zeroes added successfully earlier.  Added in a owner
    model exclusive match case in search.  Will see if it works.

  Modified on 05/01/95  3.08 by Aaron J. Martin
    Fixed some ps stuff in the response output.  Have to make sure
    that if Did > 1 that no i is printed.  Added in ps stuff, but
    havent checked it yet.

  Modified on 05/30/95  3.09 by Aaron J. Martin
    Added in some error checking for resistance values.  Checking
    a few different cases to warn user of possible problems.

  Modified on 06/13/95  3.10 by Aaron J. Martin
    Modified much of the code in resp_info.c.  Redid the way
    the table merging is done.  Cleaned up old comments.
    Added in some new routines.

  Modified on 06/29/95  3.11 by Aaron J. Martin
    Modified respest to output an estimated duration as well.
    Based for now on damping and pulse width.  Will also try
    to do all rounding calculations.

  Modified on 07/07/95  3.12 by Aaron J. Martin
    Added in a cmpflag in the merge routine to try to allow
    more thorough debugging.

  Modified on 04/02/96  3.13 by Aaron J. Martin
    Added in support for a list of table entries, via -tablist
    option.

  Modifed on 01/29/99  3.1.4 by Aaron J. Martin
    Many of the tcl/tk routines have been updated.

  Modifed on 01/29/99  4.0.0 by Aaron J. Martin
    Attempting the port to ansi C.

  Modifed on 02/08/00  4.0.1 by Aaron J. Martin
    Added in capabilities for some comments in the ps file.
    Did not add any code in to take it from the tables or anything
    yet....

  Modifed on 09/21/01  4.0.2 by Aaron J. Martin
    Added in some new color options for displaying the output.
    Just presentation stuff...

  Modifed on 03/08/02  4.0.3 by Aaron J. Martin
    Added in a help routine that spits out the table aliases.

  Modifed on 10/24/02  4.0.4 by Aaron J. Martin
    Modified the respname.tcl script to sort after globbing
    filenames.  This addresses some bugs encountered with 
    names not being in order in the directory listing.

*/
#define VERSION                      "4.0.4 10/24/02 by Aaron J. Martin (UCSB)"
static char version[64]={"@(#)RELEASE 4.0.4 10/24/02"};
