#!/bin/csh -f
# csh to run the id program and store resulting files
#
# need: pulse_file_name reftek,lpc,hpc period damping
# id.csh    step.1    reftek, 100.0 0.0 1.0 .707
#
# updated on 05/12/94 to work with version 3.0 of response
#
set usage = "$0 [-bat[ch] | -int[eractive]] [-info[ormation] infofile] [-d damping] [-f Fn] [-l lp_filt] [-h hp_filt] sacfile ..."

# path must be absolute
#set infofile = "/scec/sensor/generic"
#set infofile = "/work/aaron/larse.cal"

if ( $#argv < 1 ) then
  echo "usage: $usage"
  exit
endif

set file = $1
set idtmplog = id.log
set fulllog = id.csh.log
echo "" > $fulllog
set table = table

# set defaults
set Di = .707
set Ti = 0.21
set Hp = 0.0
set batch = 1
set getLp

set i = 1
while ( $i <= $#argv)
  switch ( $argv[$i] )
    case -bat* :
      set batch = 1 
      breaksw
    case -int* :
      set batch = 0 
      breaksw
#    case -info* :
#      @ i++
#      set infofile = $argv[$i]
#      breaksw
    case -skip :
      set skip
      breaksw
    case -d* :
      @ i++
      set Di = $argv[$i]
      breaksw
    case -f* :
      @ i++
      set Ti = $argv[$i]
      breaksw
    case -l* :
      @ i++
      set Lp = $argv[$i]
      unset getLp
      breaksw
    case -h* :
      @ i++
      set Hp = $argv[$i]
      breaksw
    default:
      # if( ! $?instart) set instart = $i
      goto process
      breaksw
    endsw
  @ i++
end

process:

#if( ! -f $infofile ) then
#  echo "cannot find info file: $infofile"
#  exit
#endif

# output comment line to table
echo "#filename               period damping idgain" >>! $table
echo "filename	Tid	Did	GNid" >>! $table
# finish on file args
while ( $i <= $#argv)
  set file = $argv[$i]
  set sn = `echo $file | nawk -F"." '{ print $5 }'`
  set comp = `echo $file | nawk -F"." '{ print $6 }'`

  if($batch) goto procfile

  echo -n "Input file $file, skip/proceed/quit (s/p/q) [p]: " 
  set tmp = $<
  if($tmp == "s") then
    @ i++
    continue
  else if($tmp == "q") then
    echo "exiting..."
    exit
  else 
    echo "processing file $file ..."
  endif

procfile:

  if(! -f $file ) continue

  # if in skip mode skip id, files are assumed to be there
  if($?skip) goto getinfo

  echo ""
  # use new respinfo prog to get starting vals
  #set Tt = `response -respinfo -single -silent -info $infofile -Tmfgr $file`
  #set Dt = `response -respinfo -single -silent -info $infofile -Dcalc $file`
  set Tt = `respest -v $file | grep period | nawk '{ print $NF}'`
  set Dt = `respest -v $file | grep damping | nawk '{ print $NF}'`

  echo "Table vals: period $Tt, Damping $Dt"

  if( $Dt == 0 ) then
    echo "zero value found, using defaults"
  else if( "$Dt" == "" ) then
    echo "empty value found, using defaults"
  else
    echo "using estimated damping: $Dt"
    set Di = $Dt
  endif
  if( $Tt == 0 ) then
    echo "zero value found, using defaults"
  else if( "$Tt" == "" ) then
    echo "empty value found, using defaults"
  else
    echo "using estimated period: $Tt"
    set Ti = $Tt
  endif

# if Lp is not set from command line, get Lp from sac header
# assume Reftek recorder, 80% of niquist (40% of sample rate)
if ( $?getLp ) then
  set Lp = `dd if=$file bs=4 count=1 | od -f | head -1 | nawk '{ printf("%.1f\n",(1/$2)*.4) }'`
    echo "using Lp (low pass filter) value derived from sac header of: $Lp"
endif


  # run id with output to screen and logfile
  echo "running id on file $file ..."
llnlid <<EOF | tee >! $idtmplog
$file
reftek $Lp $Hp 
$Ti
$Di
EOF

  # cat log
  cat $idtmplog >> $fulllog

# this is where the info is derived from the appropriate files
getinfo:

  # move resulting model and log files to correspond to input
  set idmod = ${file}.idmod
  set idlog = ${file}.idlog

  if(! $?skip) mv id_model ${idmod}
  if(! $?skip) mv $idtmplog $idlog

  # grep out needed info (last entry only)
  set To = `grep "estimated period" $idlog | tail -1 | nawk ' {print $3}'`
  set Do = `grep "estimated damping" $idlog | tail -1 | nawk ' {print $4}'`
  set Go = `grep "estimated gain" $idlog | tail -1 | nawk ' {print $3 }'`

  echo -n "$file	" >>! $table
  echo "$To	$Do	$Go" >>! $table
  @ i++
end

if( -f test ) rm test

/bin/rm -f /tmp/*.$$

# return 0 for success to rp.tcl
exit 0
